# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 打标签核心功能
# author : 
# create_time : 2020/7/1 16:33
# update_time : 2020/7/1 16:33
# copyright : Lavector
# ----------------------------------------------
import numpy as np
from lavda.util.text_util import *
from lavda.backexcutor.analysis_interface import *
from lavda.analysis.da_extract.text_format import Tradition2Simple
from lavda.analysis.da_tag.mapping_models import *
from lavda.analysis.da_tag.diatance_tag import *
from lavda.analysis.da_tag.tag_config import tag_choice_dic
from lavda.analysis.da_clean.water_spam import WaterOrdinaryWays
from lavda.config.maxda_config import *

class TextMatch(object):
    def __init__(self):
        desc = """
        ## 匹配模式
        # 	1对1，  one_match_one 词的完全匹配
        # 	1对多， one_match_multi_or 多词或者匹配
        # 	1对多， one_match_multi_and_and 多词并且包含匹配
        # 	1对多， one_match_multi_and_or  多词并且不包含匹配
        # 	1对多， one_match_multi_mix 多词并且or或者匹配

        并且 & 或者 |
        """
        print(desc)

    @staticmethod
    def one_match_one(content, line):
        content = get_Text2Formal(content)
        da_tag = get_Text2Formal(line["da_tag"])
        return da_tag if content == da_tag else False

    @staticmethod
    def one_match_multi_or(content, da_tag):
        """
        目前好像还没有需求
        """
        words = set([i for i in da_tag.split("|") if len(i) > 0])
        return True if re.search("|".join(words), content) else False
    @staticmethod
    def one_match_multi_or_clause(content, da_tag):
        """
        ”修改为了分句的一个需求
        """
        l = []
        words = set([i for i in da_tag.split("|") if len(i) > 0])
        for i in words:
            if i in content:
                l.append(i)
        return l
    @staticmethod
    def one_match_multi_and_or(content, da_tag):
        # 包含 tag1 但不包含 tag2
        tag = da_tag.split("-")
        status = various(content, tag,default_len=7)
        return status

    @staticmethod
    def one_match_multi_and_and(content, da_tag):
        # 包含 tag1 并且包含 tag2
        tag = da_tag.split("+")
        status = exclude(content, tag)
        return status

    @classmethod
    def one_match_multi_mix(cls, line, mapping, Sentment=False, choice="content"):
        """
        line: 表示每一行的数据
        mapping: 表示这个mapping文件的内容
        """
        content = get_Text2Formal(line[choice])
        channel = line['channel']
        The_Other = []
        Tag_Sen_Keword = []

        for k_map in mapping:

            Be_Label_List = []

            tag = 0

            da_tag = str(k_map[-1])
            if "-" not in da_tag and "+" not in da_tag:
                tag += cls.one_match_multi_or(content, da_tag)
                result1 = set(re.findall(da_tag, content))
                list1 = list(result1)
                Be_Label_List.extend(list1)
            else:
                for i in da_tag.split("|"):
                    if "-" in i:
                        if cls.one_match_multi_and_or(content, i):
                            if '(' in i or '（' in i:
                                i = i.split('-')[0][1:-1].split('+')
                                Be_Label_List.extend(i)
                            else:
                                Be_Label_List.append(i.split("-")[0])

                    elif "+" in i:
                        if cls.one_match_multi_and_and(content, i):
                            Be_Label_List.append(i.split("+")[0])

                    else:
                        if cls.one_match_multi_or(content, i):
                            Be_Label_List.append(i)

            if len(Be_Label_List) > 0:
                if Sentment == True:
                    # TODO 暂时注释掉
                    Sen_Collect = None  # run_aspect_sentiment(line['content'], channel, Be_Label_List)
                    Sen = Match_Sentiment(Sen_Collect)
                    print(Sen)
                    k_json = {"tag_product_1": k_map[0], "product_keyword": k_map[1], "brand_Positivity":
                        Sen['Positivity'], "brand_Neutral": Sen['Neutral'], "brand_Bad": Sen['Bad']}


                else:
                    k_json = {"tag_product_1": k_map[0], "product_keyword": k_map[1]}

                Tag_Sen_Keword.append(k_json)

        return Tag_Sen_Keword, The_Other


def Match_Sentiment(dic):
    Positivity = 0
    Neutral = 0
    Bad = 0
    for i, j in dic.items():
        Positivity += j['正向']
        Neutral += j['中性']
        Bad += j['负向']
    return {'Positivity': Positivity, 'Neutral': Neutral, 'Bad': Bad}


class TextMatch2():
    negative_word_list = WaterOrdinaryWays.get_clean('negative_word_judgment')
    goumai_list = WaterOrdinaryWays.get_clean('goumai_word_judgment')
    def __init__(self):
        self.name = None

    @staticmethod
    def is_negative_judgment(tag):
        if '@' in tag or '#' in tag:
            return True
        return False

    @staticmethod
    def negative_word_judgment(text, tag, default_length=5):
        '''
        对单个词进行否定判断
        :param text: 文本内容
        :param tag_label: 标签 @ 要否定词  #不要否定词
        :param tag: 需要判断的否定词
        :param default_length:
        :return:
        '''
        # TODO 否定词判断 就是一个最简单的否定词判断 只看前后是否有否定词然后返回
        # 否定词举例      ['莫', '未', '不怎么', '忽视', '绝无', '别说', '未必', '忘记', '放弃', '并非', '没', '从未', '不是',
        #                       '杜绝', '忌', '毫无', '绝不', '无须', '难以', '非', '无', '从来不', '不要', '从不', '不再', '绝不能',
        #                       '木有', '防止', '无法', '不会', '未尝', '永不', '切勿', '从没', '别', '决不', '不曾', '勿', '不用']
        tag_label = tag[0]
        _tag = tag[1:]
        main_index_list = index_list(text, _tag)
        for _ in main_index_list:
            status = list(filter(
                lambda x: x in text[_ - default_length if _ - default_length >= 0 else 0:_ + default_length],
                TextMatch2.negative_word_list))
            if tag_label == '@':  # 要否定词      @吸收 ---> 不吸收
                if status:
                    return True
            elif tag_label == '#':  # 不要否定词   #滋润  ---> 滋润-不滋润
                if not status:
                    return True
        return False

    @staticmethod
    def one_match_multi_or_clause(content, da_tag):
        """
        ”修改为了分句的一个需求
        """
        l = []
        words = set([i for i in da_tag.split("|") if len(i) > 0])
        for i in words:
            if TextMatch2.is_negative_judgment(i):
                if TextMatch2.negative_word_judgment(content, i):
                    l.append(i)
            elif i in content:
                l.append(i)
        return l

    # @staticmethod
    # def one_match_one(content, line):
    #     content = get_Text2Formal(content)
    #     da_tag = get_Text2Formal(line["da_tag"])
    #     return da_tag if content == da_tag else False

    @staticmethod
    def one_match_multi_or(content, da_tag):
        """
        修改增加否定词判断
        """
        words = set([i for i in da_tag.split("|") if len(i) > 0])
        for i in words:
            if TextMatch2.is_negative_judgment(i):
                if TextMatch2.negative_word_judgment(content, i):
                    return True
            else:
                # if re.search(i, content):
                if i in content:
                    return True
        return False

    @staticmethod
    def one_match_multi_and_or(content, da_tag, default_len):
        # 包含 tag1 但不包含 tag2
        tag = da_tag.split("-")
        status = various(content, tag, default_len)
        return status

    @staticmethod
    def one_match_multi_and_and(content, da_tag, default_len):
        # 包含 tag1 并且包含 tag2
        tag_list = da_tag.split("+")
        if len(tag_list) == 2:
            status = parentheses_continuous_addition_negative(content, da_tag, default_len)
        else:
            status = new_exclude(content, tag_list)
        return status

    @classmethod
    def one_match_multi_mix(cls, line, mapping, merge_obj,default_len):
        """
        line: 表示每一条记录
        mapping: 表示这个mapping文件的内容
        """
        merge_channel_list = merge_obj.channel_list

        length = mapping.length
        choice = mapping.choice
        Sentment = mapping.sentment
        Layer_number = len(mapping.mapping_list)
        sheetname = mapping.sheetname
        channel = line['channel']
        content = line['content']
        if channel in merge_channel_list:
            content = merge_format_content(line, merge_obj.filter_by(channel_name=channel), sheetname)
        content = get_Text2Formal(content)
        The_Other = []
        Tag_Sen_Keword = []

        for k_map in mapping.mapping_values:

            Be_Label_List = []

            da_tag = get_Text2Formal_keyword(k_map[-1])
            if "-" not in da_tag and "+" not in da_tag:
                Re_list = []
                words = set([i for i in da_tag.split("|") if len(i) > 0])
                for i in words:
                    if cls.is_negative_judgment(i):
                        if cls.negative_word_judgment(content, i):
                            Re_list.extend(negative_word_aspect(content,i))
                    elif i in content:
                        Re_list.append(i)
                Be_Label_List.extend(Re_list)
            else:
                for i in da_tag.split("|"):
                    if "-" in i:
                        if cls.one_match_multi_and_or(content, i,default_len):
                            tag_word = i.split("-")[0]
                            if '(' in tag_word or '（' in tag_word:
                                _ = tag_word[1:-1].split('+')
                                for j in _:
                                    if cls.is_negative_judgment(j):
                                        Be_Label_List.extend(negative_word_aspect(content, j))
                                    else:
                                        Be_Label_List.append(j)
                            else:
                                if cls.is_negative_judgment(tag_word):
                                    Be_Label_List.extend(negative_word_aspect(content, tag_word))
                                else:
                                    Be_Label_List.append(tag_word)

                    elif "+" in i and "(" not in i and "（" not in i:
                        if cls.one_match_multi_and_and(content, i,default_len):
                            for j in i.split("+"):
                                if cls.is_negative_judgment(j):
                                    Be_Label_List.extend(negative_word_aspect(content, j))
                                else:
                                    Be_Label_List.append(j)
                    else:
                        if cls.is_negative_judgment(i):
                            if cls.negative_word_judgment(content, i):
                                Be_Label_List.extend(negative_word_aspect(content, i))
                        elif i in content:
                            Be_Label_List.append(i)


            if len(Be_Label_List) > 0:

                corpora = ",".join(Be_Label_List)

                phr = Phrase()

                phr.keyword = Layer_number
                keyword = eval(phr.keyword)

                if Sentment == True:
                    str_res = keyword + "*" + corpora
                else:
                    str_res = keyword

                The_Other.append(str_res)

        return "噜".join(The_Other)
def negative_word_aspect(content,word):
    '''
    不分句打标方法用来提取否定词的aspect
    :param content: 文本
    :param word: 打标词
    :return:
    '''
    if '#' in word:
        return [word[1:]]
    elif '@' in word:
        text_list = [content[tag_index - 5 if tag_index - 5 >= 0 else 0:tag_index + len(word[1:]) + 5] for tag_index in
                     index_list(content, word[1:])]
        for _ in text_list:
            for foudingci in TextMatch2.negative_word_list:
                if foudingci in _:
                    return [foudingci, word[1:]]

class Phrase():
    @property
    def keyword(self):
        # print(self.phr)
        return self.phr

    @keyword.setter
    def keyword(self, raw):
        self.phr = phr_setter(raw)


def phr_setter(key):
    lnital = "k_map[0]"
    if key > 1:
        for i in range(1, key):
            lnital += "+'*'+k_map[{}]".format(i)
    return lnital


def new_exclude(content, tag_list):
    for tag in tag_list:
        if TextMatch2.is_negative_judgment(tag):
            if TextMatch2.negative_word_judgment(content, tag):
                tag = tag[1:]
            else:
                return False  #
        if tag not in content:
            return False
    return True


def index_list(text, word):
    return [i.start() for i in re.finditer(word, text)]


def exclude(content, tag):
    if len(tag) == 2:
        return True if re.search("(?=.*{})(?=.*{})^.*$".format(tag[0], tag[1]), content) else False
    elif len(tag) == 3:
        return True if re.search("(?=.*{})(?=.*{})(?=.*{})^.*$".format(tag[0], tag[1], tag[2]), content) else False
    elif len(tag) == 4:
        return True if re.search("(?=.*{})(?=.*{})(?=.*{})(?=.*{})^.*$".format(tag[0], tag[1], tag[2], tag[3]),
                                 content) else False
    elif len(tag) == 5:
        return True if re.search(
            "(?=.*{})(?=.*{})(?=.*{})(?=.*{})(?=.*{})^.*$".format(tag[0], tag[1], tag[2], tag[3], tag[4]),
            content) else False
    elif len(tag) == 6:
        return True if re.search(
            "(?=.*{})(?=.*{})(?=.*{})(?=.*{})(?=.*{})(?=.*{})^.*$".format(tag[0], tag[1], tag[2], tag[3], tag[4],
                                                                          tag[5]),
            content) else False
    elif len(tag) == 7:
        return True if re.search(
            "(?=.*{})(?=.*{})(?=.*{})(?=.*{})(?=.*{})(?=.*{})(?=.*{})^.*$".format(tag[0], tag[1], tag[2], tag[3],
                                                                                  tag[4], tag[5], tag[6]),
            content) else False
    elif len(tag) == 8:
        return True if re.search(
            "(?=.*{})(?=.*{})(?=.*{})(?=.*{})(?=.*{})(?=.*{})(?=.*{})(?=.*{})^.*$".format(tag[0], tag[1], tag[2],
                                                                                          tag[3], tag[4], tag[5],
                                                                                          tag[6], tag[7]),
            content) else False


def update_various_first_word(content, word, default_len):
    if "(" in word or "（" in word:
        if deal_parentheses_tag(content, word, default_len):
            return True
    else:
        if TextMatch2.is_negative_judgment(word) and TextMatch2.negative_word_judgment(content, word):
            return True
        # elif re.search(word, content):
        elif word in content:
            return True


def replace_parentheses(words):
    rep = {'(': '', ')': '', '（': "", '）': ''}
    rep = dict((re.escape(k), v) for k, v in rep.items())
    pattern = re.compile("|".join(rep.keys()))
    my_str = pattern.sub(lambda m: rep[re.escape(m.group(0))], words)
    return my_str

def parentheses_continuous_addition_negative(content, tag, preset_length=7):
    '''
    重写的 A+B 的判断规则 增加了否定词的判断
    :param content:
    :param tag:
    :param preset_length:
    :return:
    '''
    assert type(preset_length) == int, "长度不符合规范"
    tag_list = tag.split("+")
    assert len(tag_list) == 2, "关键词长度不符合规范"
    first_word = tag_list[0]
    end_word = tag_list[1]
    _first_word = _end_word = ''
    if TextMatch2.is_negative_judgment(first_word):
        if TextMatch2.negative_word_judgment(content, first_word):
            _first_word = first_word[1:]
        else:
            return False
    if TextMatch2.is_negative_judgment(end_word):
        if TextMatch2.negative_word_judgment(content, end_word):
            _end_word = end_word[1:]
        else:
            return False
    a = first_word if not _first_word else _first_word
    b = end_word if not _end_word else _end_word
    # if not re.search("(?=.*{})(?=.*{})^.*$".format(a, b), content):
    ab = a in content and b in content
    if not ab:
        return False
    first_length = len(a)
    second_length = len(b)
    first_index = index_list(content, a)
    second_index = index_list(content, b)
    for first in first_index:
        for second in second_index:
            if first < second:
                length = second - (first + first_length)
            else:
                length = first - (second + second_length)
            if length < preset_length:
                return True
    return False


def parentheses_continuous_addition(content, tag, preset_length=7):
    assert type(preset_length) == int, "长度不符合规范"
    tag_list = tag.split("+")
    if not re.search("(?=.*{})(?=.*{})^.*$".format(tag_list[0], tag_list[1]), content):
        return False
    first_word = tag_list[0]
    end_word = tag_list[1]
    first_length = len(first_word)
    second_length = len(end_word)
    first_index = indexMany(content, first_word)
    second_index = indexMany(content, end_word)
    for first in first_index:
        for second in second_index:
            if first < second:
                length = second - (first + first_length)
            else:
                length = first - (second + second_length)
            if length < preset_length:
                return True
    return False


def deal_parentheses_tag(content, tag_words, default_len):
    parentheses_words = replace_parentheses(tag_words)
    return parentheses_continuous_addition_negative(content, parentheses_words, default_len)


def various(content, tag, default_len):
    count = 0
    if update_various_first_word(content, tag[0], default_len):
        for i in tag[1:]:
            if "(" in i or "（" in i:

                if deal_parentheses_tag(content, i, default_len):
                    count += 1
            else:
                if i in content:
                    count += 1
        if count == 0:
            return True
        else:
            return False
    else:
        return False


def merge_content(line):
    content = "" if not line['content'] else line['content']
    title = "" if not line['title'] else line['title']
    result = str(content) + str(title)
    return result


def get_Text2Formal(line):
    """
    1. 将繁体转换成简体
    2. 删除表情
    3. \n 转化为空
    4. !！?？#,，@… ~～。. 全部用空格替换
    5. 将2个或者以上空格转化为空
    6. 将 字母+空格(一个或多个)+字母 的空格转化为_，并将字母转化为小写
    """
    line = str(line)
    # t2s = Tradition2Simple()
    # line = t2s.process(line)
    line = remove_emoji(line)
    line = re.sub(r"[\n]+", "", line.strip())
    line = re.sub(r"[!！?？#,，@… ~～。.`、'‘]+", " ", line)
    line = re.sub(r"( ){2,}", "", line)
    while 1:
        result = re.sub(r'(.*[A-Za-z])( )+([A-Za-z].*)', r'\1_\3', line)
        if result == line:
            break
        line = result
    return line.lower().strip()
def get_Text2Formal_keyword(line):
    """
    只是针对于关键词  与 get_Text2Formal 相比  去除了将 @，# 变为空格的规则，其他不变
    1. 将繁体转换成简体
    2. 删除表情
    3. \n 转化为空
    4. !！?？,，… ~～。. 全部用空格替换
    5. 将2个或者以上空格转化为空
    6. 将 字母+空格(一个或多个)+字母 的空格转化为_，并将字母转化为小写
    """
    line = str(line)
    # t2s = Tradition2Simple()
    # line = t2s.process(line)
    line = remove_emoji(line)
    line = re.sub(r"[\n]+", "", line.strip())
    line = re.sub(r"[!！?？,，… ~～。.`、'‘]+", " ", line)
    line = re.sub(r"( ){2,}", "", line)
    while 1:
        result = re.sub(r'(.*[A-Za-z])( )+([A-Za-z].*)', r'\1_\3', line)
        if result == line:
            break
        line = result
    return line.lower().strip()


def conduct_relation_tag_tree(data, mapping):
    data['merge_tag'] = data.apply(lambda line: deal_sentence_tag(line, mapping), axis=1)
    data['merge_index'] = range(data.shape[0])
    split_merge_index = data.groupby('merge_index')
    df_list = [related_word_mapping(the_data, mapping) for name, the_data in split_merge_index]
    df_data = pd.concat(df_list, join="inner")
    df_data.drop(['merge_index', 'merge_tag'], axis=1, inplace=True)
    return df_data


def related_word_mapping(intermediate, line, mapping):
    merge_series = line[intermediate]
    if str(merge_series.values[0]) in ["nan", "", "NaN", "null", "Null", "None"]:
        return line
    merge_series_values = eval(merge_series.values[0])
    contrast_list = merge_series_values[0]
    the_now_list = [re.findall("(?<=\()\S+(?=\))", contra)[0] for contra in contrast_list]

    merge_list = np.array(merge_series_values)

    # the_mapping_list = mapping.get_map_name_collect()
    inner_name = [name + "-id" for name in the_now_list]

    total_merge_df = pd.DataFrame(merge_list, columns=inner_name)
    for _sheet in the_now_list:
        sheet_name = _sheet
        sheet_df = mapping.filter_by(name=_sheet).mapping_df
        total_merge_df["{}-id".format(sheet_name)] = total_merge_df["{}-id".format(
            sheet_name)].apply(lambda line: -1 if line == "({})".format(sheet_name) else int(line.split("(")[0]))
        total_merge_df = pd.merge(total_merge_df, sheet_df, on="{}-id".format(sheet_name), how="left")
    total_merge_df = total_merge_df.drop(inner_name, axis=1)
    line.insert(1, 'related_index', [1])
    total_merge_df['related_index'] = 1
    inner_line = pd.merge(line, total_merge_df, on="related_index", how='left')
    line = inner_line.drop(columns=['related_index'])
    return line


def deal_sentence_tag(line, mapping, map_1=None, map_0=None):
    # content = "1葡萄牙和中国是芯动的感觉！2葡萄牙和中国是芯动的感觉，韩硕和韩松林端午一起去了长城。3京东618限时定制一款兰博基尼！4京东618限时定制一款兰博基尼，然后韩硕和韩松林和韩翱键盘端午一起去了长城！5京东618限时定制一款怀旧版得兰博基尼，将由葡萄牙和中国一起生产，然后韩硕和韩松林和韩翱键盘端午一起去了长城！"
    test_tag = SplitSentenceTag()
    result = test_tag.process(line, mapping, map_1, map_0)
    result1 = result[0]  # 之前的打标
    result2 = result[-1]  # 新加的中间结果
    return result1, result2


class SplitSentenceTag(object):
    def __init__(self, params=None):
        pass

    def process(self, line, mapping, map_1=None, map_0=None):
        '''
        原来的话在这设了个列表用来接收，现在计划返回一个列表
        :param line:    一行数据
        :param mapping:    关键词表对象
        :return:
        '''
        sentences = extract_content(line, mapping)
        # print(sentences)
        list_all_sentences_tag = []
        list_all_data = []  # 存放新增需求信息：
        for content_i in sentences:
            # 对每个分句进行打标
            result = get_relation_tag(line,content_i, mapping, map_1, map_0)  # 打标返回结果的要求：
            # 对打标结果进行拆分
            list_tag_int = tag_split(result[0])
            list_all_sentences_tag.extend(list_tag_int)
            list_all_data.extend(result[-1])

        # 对一句话中的标签进行去重。 下面的双重循环去不掉重复的数据。k != i
        new_list_all_sentences_tag = []
        for list_i in list_all_sentences_tag:
            if list_i not in new_list_all_sentences_tag:
                new_list_all_sentences_tag.append(list_i)

        # 将list放入一个新list地址中， 避免删除new_list_all_sentences_tag第一个数据时，遍历不到第二个数据得问题。
        new_list_all_sentences_tag_last = []
        new_list_all_sentences_tag_last = new_list_all_sentences_tag_last + new_list_all_sentences_tag
        for i in new_list_all_sentences_tag:
            # 当打标表为一个词表时 新加的 处理翱建遗留的bug
            if type(i) == str:
                continue
            for str_j in [k for k in new_list_all_sentences_tag if k != i]:
                if (len(i) == 2) and (i[0] in str_j) and (i[1] in str_j) and (i in new_list_all_sentences_tag_last):
                    new_list_all_sentences_tag_last.remove(i)
                elif (len(i) == 3) and (i[0] in str_j) and (i[1] in str_j) and (i[2] in str_j) and (
                        i in new_list_all_sentences_tag_last):
                    new_list_all_sentences_tag_last.remove(i)
                elif (len(i) == 4) and (i[0] in str_j) and (i[1] in str_j) and (i[2] in str_j) and (i[3] in str_j) and (
                        i in new_list_all_sentences_tag_last):
                    new_list_all_sentences_tag_last.remove(i)
                elif (len(i) == 5) and (i[0] in str_j) and (i[1] in str_j) and (i[2] in str_j) and (i[3] in str_j) and (
                        i[4] in str_j) and (i in new_list_all_sentences_tag_last):
                    new_list_all_sentences_tag_last.remove(i)
                elif (len(i) == 6) and (i[0] in str_j) and (i[1] in str_j) and (i[2] in str_j) and (i[3] in str_j) and (
                        i[4] in str_j) and (i[5] in str_j) and (i in new_list_all_sentences_tag_last):
                    new_list_all_sentences_tag_last.remove(i)
                elif (len(i) == 7) and (i[0] in str_j) and (i[1] in str_j) and (i[2] in str_j) and (i[3] in str_j) and (
                        i[4] in str_j) and (i[5] in str_j) and (i[6] in str_j) and (
                        i in new_list_all_sentences_tag_last):
                    new_list_all_sentences_tag_last.remove(i)

        # 如果长度大于2 ,里面是空的去掉。
        # if len(new_list_all_sentences_tag_last) >= 2:
        #     for list_small in new_list_all_sentences_tag_last:
        #         if set(list_small) == set(['']):
        #             new_list_all_sentences_tag_last.remove(list_small)

        if len(new_list_all_sentences_tag_last) >= 2:
            for list_small in new_list_all_sentences_tag_last:
                str_j = "".join(list_small).strip()
                if not bool(re.search(r'\d', str_j)):
                    new_list_all_sentences_tag_last.remove(list_small)

        return new_list_all_sentences_tag_last, list_all_data


def tag_split(tag_dict):
    dict_to_list = list(tag_dict.values())
    dict_count = len(tag_dict)
    tag_all_list = []
    if dict_count == 1:
        tag_all_list = dict_to_list[0]
    elif dict_count == 2:
        for i in dict_to_list[0]:
            for j in dict_to_list[1]:
                tag_all_list.append([i, j])
    elif dict_count == 3:
        for i in dict_to_list[0]:
            for j in dict_to_list[1]:
                for k in dict_to_list[2]:
                    tag_all_list.append([i, j, k])
    elif dict_count == 4:
        for i in dict_to_list[0]:
            for j in dict_to_list[1]:
                for k in dict_to_list[2]:
                    for l in dict_to_list[3]:
                        tag_all_list.append([i, j, k, l])
    elif dict_count == 5:
        for i in dict_to_list[0]:
            for j in dict_to_list[1]:
                for k in dict_to_list[2]:
                    for l in dict_to_list[3]:
                        for m in dict_to_list[4]:
                            tag_all_list.append([i, j, k, l, m])
    elif dict_count == 6:
        for i in dict_to_list[0]:
            for j in dict_to_list[1]:
                for k in dict_to_list[2]:
                    for l in dict_to_list[3]:
                        for m in dict_to_list[4]:
                            for n in dict_to_list[5]:
                                tag_all_list.append([i, j, k, l, m, n])
    elif dict_count == 7:
        for i in dict_to_list[0]:
            for j in dict_to_list[1]:
                for k in dict_to_list[2]:
                    for l in dict_to_list[3]:
                        for m in dict_to_list[4]:
                            for n in dict_to_list[5]:
                                for o in dict_to_list[6]:
                                    tag_all_list.append([i, j, k, l, m, n, o])

    return tag_all_list

def remove_topic(content):
    '''
    去除文本中的话题，@后的内容
    :param content: 文本内容
    :return:
    '''
    if '@' in content:
        # 正常的@用户
        li = re.findall(r'@(.*?) ', content)
        _ = ['@' + i for i in li]
        for i in _:
            if len(i) > 17:
                i = i[:17]
            content = content.replace(i, '')
        # 不正常的@用户
        aite_index = index_list(content, '@')
        _ = [content[i:i + 8] for i in aite_index]
        for i in _:
            content = content.replace(i, '')
    if '#' in content:
        a = re.findall(r'#(.*?)#', content)
        _ = ['#' + i + '#' for i in a]
        for i in _:
            content = content.replace(i, '')
    return content


def extract_content(line, mapping):
    '''
    判断是否需要合并字段，返回要打标的内容
    :param line: 一行数据
    :return:
    '''
    max_list = []
    content_list = []  # 存放要打标的内容
    default_value = 'content'
    for map_i in mapping:
        if line['channel'] in tag_choice_dic and '*' in tag_choice_dic[line['channel']].keys():
            max_list.extend(tag_choice_dic[line['channel']]['*'])
        elif line['channel'] in tag_choice_dic and map_i.sheetname in tag_choice_dic[line['channel']]:
            max_list.extend(tag_choice_dic[line['channel']][map_i.sheetname])
    max_list = list(set(max_list))

    if max_list:
        for i in max_list:
            sentences = re.split(split_sign, remove_topic(line[i]))
            for j in sentences:
                content_dict = {}    # 存放要合并的内容
                content_dict[i] = get_Text2Formal(j)
                content_list.append(content_dict)
        return content_list
    else:
        sentences = re.split(split_sign, remove_topic(line[default_value]))
        for j in sentences:
            content_dict = {}    # 存放要合并的内容
            content_dict[default_value] = get_Text2Formal(j)
            content_list.append(content_dict)
        return content_list


def dabiaofunc(name, map_i, map_values, value, result_list, map_list, default_len=7):
    # TODO 打标规则 -------------------
    # 后来加的购买 ，不太合适
    if name == '购买' and len(list(filter(lambda x: x in value, TextMatch2.goumai_list))) == 0:
        return result_list, map_list

    for map_res in map_values:
        map_ = map_res[-1]
        tag = 0
        if "-" not in map_ and "+" not in map_:
            if TextMatch2.one_match_multi_or(value, map_):
                tag += 1
                d = map_i.mapping_df[
                    map_i.mapping_df['{}(keyword)'.format(name)] == map_].values  # 包含父级关系的数据 列表
                dd = '~'.join(map(str, d[0][:-1]))
                result = TextMatch2.one_match_multi_or_clause(value, map_)
                result_list.append(dd + '*' + map_ + '*' + ','.join(result))
        else:
            for i in map_.split("|"):
                if "-" in i:
                    if TextMatch2.one_match_multi_and_or(value, i, default_len):
                        tag += 1
                        d = map_i.mapping_df[map_i.mapping_df['{}(keyword)'.format(name)] == map_].values
                        dd = '~'.join(map(str, d[0][:-1]))
                        if "(" not in i and "（" not in i:
                            result_list.append(dd + '*' + map_ + '*' + i.split('-')[0])
                        else:
                            result = replace_parentheses(i.split('-')[0])
                            result_list.append(dd + '*' + map_ + '*' + ','.join(result.split('+')))

                elif "+" in i and "(" not in i and "（" not in i:
                    if TextMatch2.one_match_multi_and_and(value, i, default_len):
                        tag += 1
                        d = map_i.mapping_df[map_i.mapping_df['{}(keyword)'.format(name)] == map_].values
                        dd = '~'.join(map(str, d[0][:-1]))
                        result_list.append(dd + '*' + map_ + '*' + ','.join(i.split('+')))
                else:
                    if TextMatch2.one_match_multi_or(value, i):
                        tag += 1
                        d = map_i.mapping_df[map_i.mapping_df['{}(keyword)'.format(name)] == map_].values
                        dd = '~'.join(map(str, d[0][:-1]))
                        result_list.append(dd + '*' + map_ + '*' + i)
        if tag > 0:
            map_list.append("{}({})".format(str(map_res[0]), name))
    return result_list, map_list


def get_relation_tag(line, content, relation_mapping, map_1=None, map_0=None,):
    mappings = relation_mapping
    result_dic = {}
    result_list = []  # 存放关键词和实际打到的词
    # 我给的content是： {content or title:'要打标的文本内容'}
    key = list(content.keys())[0]
    value = list(content.values())[0]
    for map_i in mappings:
        map_list = []
        name = map_i.sheetname
        # 因为只有品牌有title要求，其他都没有。
        if name == '品牌':
            if '品牌(需求)' in map_i.mapping_list:  # 填充情况下的词表
                map_values_jh = map_1[['{}-id'.format(name), '{}(keyword)'.format(name)]].values
                if key == 'title' and line['channel'] in ['tmall', 'jd']:
                    result_list, map_list = dabiaofunc(name, map_i, map_values_jh, value, result_list, map_list,
                                                       default_len=100)
                else:
                    result_list, map_list = dabiaofunc(name, map_i, map_values_jh, value, result_list, map_list)
                if len(map_list) == 0:
                    map_values_jh = map_0[['{}-id'.format(name), '{}(keyword)'.format(name)]].values
                    result_list, map_list = dabiaofunc(name, map_i, map_values_jh, value, result_list, map_list)
            else:  # 正常情况下的词表
                map_values_jh = map_i.mapping_df[['{}-id'.format(name), '{}(keyword)'.format(name)]].values
                if key == 'title' and line['channel'] in ['tmall', 'jd']:
                    result_list, map_list = dabiaofunc(name, map_i, map_values_jh, value, result_list, map_list,
                                                       default_len=100)
                else:
                    result_list, map_list = dabiaofunc(name, map_i, map_values_jh, value, result_list, map_list)
        else:
            if line['channel'] in tag_choice_dic and '*' in tag_choice_dic[line['channel']].keys() and key in \
                    tag_choice_dic[line['channel']]['*']:
                map_values_jh = map_i.mapping_df[['{}-id'.format(name), '{}(keyword)'.format(name)]].values
                result_list, map_list = dabiaofunc(name, map_i, map_values_jh, value, result_list, map_list)
            elif key == 'content':
                map_values_jh = map_i.mapping_df[['{}-id'.format(name), '{}(keyword)'.format(name)]].values
                result_list, map_list = dabiaofunc(name, map_i, map_values_jh, value, result_list, map_list)
        if len(map_list) > 0:
            result_dic[name] = map_list
        else:
            result_dic[name] = ['({})'.format(name)]
    return result_dic, result_list


class TagContentChoiceCollect:
    tag_content_choice_list = []

    def __init__(self, choice_dict_collect):
        self.tag_content_choice_list = [TagContentChoice({channel: choice})
                                        for channel, choice in choice_dict_collect.items()]

    @property
    def channel_list(self):
        return [tag_content_choice.channel for tag_content_choice in self.tag_content_choice_list]

    def filter_by(self, **kwargs):
        channel_name = kwargs.get('channel_name')
        filter_tag_choice = list(filter(lambda tag_content_choice: tag_content_choice.channel == channel_name,
                                        self.tag_content_choice_list))
        if len(filter_tag_choice) > 0:
            result = filter_tag_choice[0]
        else:
            result = None
        return result


class TagContentChoice:
    def __init__(self, choice_dict):
        self.channel = list(choice_dict.keys())[0]
        self.choice = choice_dict.get(self.channel)

    @property
    def deal_sheet(self):
        return list(self.choice.keys())

    def merge_content_list(self, sheet):
        # 这个if判断是贾浩后来加上的
        if None == self.choice.get(sheet):
            return self.choice['*']
        return self.choice.get(sheet)


def merge_format_content(line, merge_obj, sheetname):
    # 第一个if判断是贾浩后来加上的
    if '*' in merge_obj.deal_sheet:
        merge_list = merge_obj.merge_content_list(sheetname)
        merge_text_list = [deal_format_nan_text(line[text]) for text in merge_list]
        result = "".join(merge_text_list)
    elif sheetname in merge_obj.deal_sheet:
        merge_list = merge_obj.merge_content_list(sheetname)
        merge_text_list = [deal_format_nan_text(line[text]) for text in merge_list]
        result = "".join(merge_text_list)
    else:
        result = str(line['content'])
    return result


def deal_format_nan_text(text):
    text = "" if not text else str(text)
    return text


# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 业务级标签/基于算法的打标签功能
# author : LGD
# create_time : 2021/2/22 16:32
# update_time : 2021/2/22 16:32
# copyright : Lavector
# ----------------------------------------------
import os

from lavda.util.mysql_util import CommonLavDbReader
from lavda.config.maxda_config import LAV_AI_DB_CONFIG
from lavda.analysis.da_tag.label_model import LabelComponentModel


class LabelResult(object):
    YES = 1
    UN_CERTAIN = 0
    NO = -1


class LabelCheckInterface(object):
    def predict(self, content, aspect, start, end):
        """

        :param content: string, 文本内容
        :param aspect: string, 需要判断的aspect词
        :param start: int, aspect的开始位置
        :param end: int, aspect的结束位置
        :return: bool
        """
        pass

    def predict_list(self, content, aspect_list):
        """

        :param content:
        :param aspect_list: list(dict)
        :return:
        """
        # 添加id字段
        id_aspect_list = list()

        id_i = 0
        for item in aspect_list:
            id_aspect_list.append([id_i, item])
            id_i += 1

        # 排序
        sort_aspect = sorted(id_aspect_list, key=lambda x: x[-1]['start'])

        # 1.分组
        i = 0
        id_group_dict = dict()
        while i < len(sort_aspect):
            if i == 0:
                # group id 从0开始
                id_group_dict[sort_aspect[i][0]] = 0
            else:
                last_item = sort_aspect[i-1][-1]
                last_item_id = sort_aspect[i-1][0]
                cur_item = sort_aspect[i][-1]
                cur_item_id = sort_aspect[i][0]

                last_end = last_item['end']
                cur_start = cur_item['start']
                if cur_start - last_end <= 1 and content[last_end] in {"+", '和', '、', "与"}:
                    id_group_dict[cur_item_id] = id_group_dict[last_item_id]
                else:
                    # 开启新的id
                    id_group_dict[cur_item_id] = id_group_dict[last_item_id] + 1
            i += 1

        # 2.预测结果
        group_result_dict = dict()
        for cur_id, item in sort_aspect:
            one_result = self.predict(content, item['aspect'], item['start'], item['end'])
            item['aspectCategory'] = one_result
            # if one_result:
            #     item['aspectCategory'] = True
            # else:
            #     item['aspectCategory'] = False

            cur_group_id = id_group_dict[cur_id]
            if cur_group_id not in group_result_dict:
                group_result_dict[cur_group_id] = LabelResult.UN_CERTAIN

            if item['aspectCategory'] != LabelResult.UN_CERTAIN:
                group_result_dict[cur_group_id] = item['aspectCategory']

        # 3.根据同组的
        for cur_id, item in sort_aspect:
            cur_group_id = id_group_dict[cur_id]

            cur_group_result = group_result_dict[cur_group_id]
            if item['aspectCategory'] != cur_group_result:
                item['aspectCategory'] = cur_group_result

        result = [item[-1] for item in sort_aspect]

        return result


class LabelComponentChecker(object):
    """
    成分标签检测
    """
    def __init__(self, root_model_path='/data/share/nlp',
                 use_dict=True):
        """

        :param root_model_path: string,模型根路径
        :param use_dict: bool, 是否使用词典判断
        """
        self.model = LabelComponentModel(os.path.join(root_model_path, *['label', "component"]))

        if use_dict:
            df_word = CommonLavDbReader.get_as_df("pro_label_dict", db_config=LAV_AI_DB_CONFIG)
            word_list = df_word.to_dict(orient="records")
            self.must_word_set = set([item["term"] for item in word_list if item['category'] == '成分'])
        else:
            self.must_word_set = set()

    def predict(self, content, aspect_list):
        """

        :param content: string,
        :param aspect_list: list(item)
                eg: [{'aspect': '中国', 'start': 0, 'end': 3}]
        :return:
        """
        index2result = [False] * len(aspect_list)

        # 1.基于词典匹配
        tmp_list = list()
        tmp_index_list = list()
        for i, item in enumerate(aspect_list):
            if item['aspect'] in self.must_word_set:
                index2result[i] = True
            else:
                tmp_list.append(item)
                tmp_index_list.append(i)

        # 2.基于模型进行判断
        format_input = [[content, ii_item['aspect'], ii_item['start'], ii_item['end']] for ii_item in
                        tmp_list]
        if len(format_input) > 0:
            predict_result = self.model.predict_batch(format_input)
            for i, one_result in enumerate(predict_result):
                old_index = tmp_index_list[i]
                index2result[old_index] = one_result

        # 3.检查并处理并列结构
        if False in set(index2result):
            index2result = self.merge_by_group(content, aspect_list, index2result)

        return index2result

    def merge_by_group(self, content, aspect_list, index2result):
        """
        检查并列结构的数据，并将其保持一致
        :param aspect_list:
        :param index2result:
        :return:
        """
        # 添加id，id和index保持一致
        id_aspect_list = list()

        for i, item in enumerate(aspect_list):
            id_aspect_list.append([i, item])

        # 排序
        sort_aspect = sorted(id_aspect_list, key=lambda x: x[-1]['start'])

        # 1.分组
        i = 0
        # item id => group id
        id_group_dict = dict()
        while i < len(sort_aspect):
            if i == 0:
                # group id 从0开始
                id_group_dict[sort_aspect[i][0]] = 0
            else:
                last_item = sort_aspect[i - 1][-1]
                last_item_id = sort_aspect[i - 1][0]
                cur_item = sort_aspect[i][-1]
                cur_item_id = sort_aspect[i][0]

                last_end = last_item['end']
                cur_start = cur_item['start']
                if 0 < cur_start - last_end <= 1 and content[last_end] in {"+", '和', '、', "与"}:
                    id_group_dict[cur_item_id] = id_group_dict[last_item_id]
                else:
                    # 开启新的id
                    id_group_dict[cur_item_id] = id_group_dict[last_item_id] + 1
            i += 1

        # 2.预测结果
        group_result_dict = dict()
        for cur_id, item in sort_aspect:
            cur_group_id = id_group_dict[cur_id]
            if cur_group_id not in group_result_dict:
                group_result_dict[cur_group_id] = False

            # TODO 如果该组为True 即表示为标签，则不再处理了
            if group_result_dict[cur_group_id]:
                continue
            group_result_dict[cur_group_id] = index2result[cur_id]

        # 3.根据同组的
        for cur_id, item in sort_aspect:
            cur_group_id = id_group_dict[cur_id]

            # TODO 如果为True 即表示为标签，则不再处理了
            if index2result[cur_id]:
                continue

            index2result[cur_id] = group_result_dict[cur_group_id]

        return index2result


if __name__ == '__main__':
    com_checker = LabelComponentChecker(use_dict=False)
    content = """细腻，维E和透明质酸钠等成分有效保湿及养护唇部肌肤。丝滑细腻，维E和透明质酸钠等成分有效保湿及养护唇部肌肤。"""
    aspect_list = [{'aspect': '维E', 'start': 3, 'end': 5},
                   {'aspect': '透明质酸钠', 'start': 6, 'end': 11},
                   {'aspect': '维E', 'start': 31, 'end': 33},
                   {'aspect': '透明质酸钠', 'start': 34, 'end': 39},
                   ]

    # print(content[3: 5], content[6: 11], content[31: 33], content[34: 39])
    result = com_checker.predict(content, aspect_list)
    print(result)




# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 数据质量检测
# author :
# create_time : 2020/7/15 16:28
# update_time : 2020/7/15 16:28
# copyright : Lavector
# ----------------------------------------------
import pandas as pd
import numpy as np
import datetime
from lavda.backexcutor.analysis_interface import *
from lavda.analysis.common import parse_multi_column


class QaCommonWays(object):
    @staticmethod
    def save(all_df_result_dict, result_file_path):
        writer = pd.ExcelWriter(result_file_path)
        for label_name in all_df_result_dict:
            df_one_label = all_df_result_dict[label_name]
            df_one_label.to_excel(writer, sheet_name=label_name, index=True)
        writer.save()

    @staticmethod
    def count_percent(df_result, df_data_count, cul_name, name_count, name_percent):
        df_count = df_result.groupby([cul_name]).agg({cul_name: 'count'})
        df_count.rename(columns={cul_name: name_count}, inplace=True)
        df_count.sort_values(by=[name_count], axis=0, ascending=False, inplace=True)
        # 比例
        df_count[name_percent] = df_count[name_count].astype(float)/df_data_count.astype(float)
        df_count[name_percent] = df_count[name_percent].apply(lambda x: '%.2f%%' % (x*100))
        return df_count

    @staticmethod
    def get_count_column(df_result, prefix):
        column_name = df_result.columns.tolist()
        # print(column_name)
        count_column_list = []
        for i in column_name:
            # print(i)
            if i.startswith(prefix):
                count_column_list.append(i)
        return count_column_list

    @staticmethod
    def clean_count(df_result, prefix):
        # 获取要统计的列名
        get_count_column_list = QaCommonWays.get_count_column(df_result, prefix)
        list_front = ["清洗前数据总量"]
        list_after = ["清洗后数据总量"]
        list_front.extend(get_count_column_list)
        list_front.extend(list_after)
        # print(list_front)
        dict_count = {'channel': list_front}

        # 分组统计各个渠道的数据
        df_grouped = df_result.groupby(df_result['channel'])
        for name, df_group in df_grouped:
            count_0 = [df_group.shape[0]]
            tag_list = df_group[get_count_column_list].values.astype(int).tolist()
            # 各个渠道1的求和
            list_1_sum = np.sum(tag_list, axis=0).tolist()
            count_0.extend(list_1_sum)

            # 将数据清洗前后的数据量加进去
            list_0_sum = np.sum(tag_list, axis=1).tolist()
            count = 0
            for i in list_0_sum:
                if i == 0:
                    count += 1
            count_1 = [count]
            count_0.extend(count_1)
            dict_count[name] = count_0
        df_count = pd.DataFrame(dict_count)
        return df_count


class PreQuality(DfStatisInterface):
    """
    离散型数据质量统计
    即统计各个值的数量，
    """
    def __init__(self, index_column, input_column_str):
        """

        :param index_column: string， 数据索引列名
        :param input_column_str: string， 待统计的数据列名
                                eg: content, title
        """
        self.index_column = index_column
        self.count_cul_name_list = parse_multi_column(input_column_str)

    def process(self, data, **params):
        """
        :param data: 输入数据 df
        :param params:
        :return:
        """
        qa_common_friend = QaCommonWays()
        df_data_count = data[self.index_column].count()

        column_set = set(data.columns.tolist())

        all_df_result_dict = dict()
        for name_i in self.count_cul_name_list:
            if name_i not in column_set:
                continue
            cul_name_count = qa_common_friend.count_percent(data, df_data_count, name_i, "%s_count" % (name_i), "%s_percent" % (name_i))
            all_df_result_dict["%s_count" % name_i] = cul_name_count

        return all_df_result_dict


class CleanQuality(DfStatisInterface):
    """
    清洗后数据的质量
    """
    def __init__(self, prefix="tag_clean"):
        """
        清洗结果字段的前缀
        :param prefix:
        """
        self.prefix = prefix

    def process(self, df_result, params=None):
        """
        params:输出路径
         """
        qa_common_friend = QaCommonWays()

        df_count = qa_common_friend.clean_count(df_result, self.prefix)

        return df_count


class TagQuality(DfStatisInterface):
    """
    统计标签的质量
    """
    def __init__(self, index_column, input_column_prefix):
        """
        :param input_column_prefix: 需要统计字段前缀
        """
        self.index_column = index_column
        self.input_column_prefix = input_column_prefix.lower()

    def process(self, data, params=None):
        column_name = data.columns.tolist()
        column_name = [name.lower() for name in column_name]
        input_column_list = [name for name in column_name if name.startswith(self.input_column_prefix)]
        data['datetime_year_month'] = data['datetime'].astype(str).apply(lambda line: line[0:7])

        all_df_result_dict = dict()
        # 提取各个sheet需要统计的字段。
        for input_name in input_column_list:
            last_count_col_name_list = ['datetime_year_month']

            if input_name not in column_name:
                continue
            last_count_col_name_list.append(input_name)

            dup_list = [self.index_column]
            dup_list.append(input_name)
            data_dup = data.drop_duplicates(subset=dup_list, keep='first')

            # 开始对各个sheet的标签统计
            df_tag_count = data_dup.groupby(last_count_col_name_list).agg({input_name: 'count'})
            # df_tag_count = data.groupby(last_count_col_name_list).agg(['count'])
            # df_tag_count.rename(columns={(input_name: '%s_keywords_count' % (input_name)}, inplace=True))
            df_tag_count.rename(columns={input_name: '%s_count' % (input_name)},
                                inplace=True)

            df_tag_count = df_tag_count.reset_index()
            all_df_result_dict[input_name] = df_tag_count

        # 数据多sheet保存
        return all_df_result_dict


def main():
    infile1 = "C:/Users/zhang/Desktop/data_teest.xlsx"
    outfile = "C:/Users/zhang/Desktop/test666.xlsx"
    df_tag = pd.read_excel(infile1, header=0, error_bad_lines=False, encoding="utf_8_sig")

    # 预处理代码测试
    # count_cul_name_list = ['age', 'gender', 'city', 'city_rank', 'age_level']
    # qa = PreQuality(count_cul_name_list)
    # qa.process(df_tag, outfile)

    # 打标质量检测代码测试
    wayfriend = QaCommonWays()
    qa = TagQuality('da_index','tag_format')
    dict_test = qa.process(df_tag)
    wayfriend.save(dict_test,outfile)


    # 清洗质量检测代码测试
    # qa = CleanQuality()
    # qa.process(df_tag, outfile)


if __name__ == "__main__":
    # 开始时间
    sta_time = datetime.datetime.now()

    main()

    # 结束时间
    end_time = datetime.datetime.now()
    times = (end_time - sta_time).seconds
    print("\n运行时间: %ss == %sm == %sh\n\n" % (times, times/60, times/60/60))
    print('文件存储成功！')


# -*- coding: utf-8 -*-
import os

import pandas as pd
from lavda.util.panda_tool import PandaTool
from lavda.util.file_tool import FileTool

from lavda.backexcutor.analysis_interface import *
from lavda.analysis.da_clean.df_distinct import DfDistinct

from lavda.backexcutor.task import TaskBuilder


class BaseExecutor(object):
    def __init__(self, channel_col="channel", max_lines=700000):
        self._task_meta = None
        self._max_lines = max_lines
        self._channel_col = channel_col

    @property
    def task_meta(self):
        return self._task_meta

    @task_meta.setter
    def task_meta(self, value):
        self._task_meta = value

    @property
    def max_lines(self):
        return self._max_lines

    @max_lines.setter
    def max_lines(self, value):
        self._max_lines = value

    @property
    def channel_col(self):
        return self._channel_col

    @channel_col.setter
    def channel_col(self, value):
        self._channel_col = value

    def run_flow(self,
                 config_flow,
                 input_source,
                 config_data_path,
                 output_source=None,
                 statis_output_path=None,
                 clean_result_prefix="tag_clean_"):
        """
        运行数据处理任务流
        :param config_flow: string or dict,配置文件或配置数据
                            (1) string:任务流配置文件
                            (2) dict：基于配置文件解析出来的任务流数据
        :param input_source: string, 输入数据, 可选值:
                                （1）文件目录： 此目录中所有的以suffix为后缀的文件都作为输入数据
                                 (2) 文件（須包含完整的路径）：单个文件作为数据
                                 (3) 文件列表（须包含完整的路径）：list/tuple(文件)
                                 （4） HiveSource
        :param config_path: string, 配置数据目录, 必须是已经存在的目录
                             此目录用以存放各个Task所需要的资源，
        :param output_source: 数据输出
                          (1)string,数据处理结果文件名,必须是包含完整路径的文件名
                            当最后处理结果一个文件存不下时，必自动存储为多个文件
                            以输入的文件名作为前缀
                                eg: /Download/output.xlsx, 实际结果可以为 output_1.xlsx, output_2.xlsx
                          （2）HiveSource
        :param statis_output_path: 统计类结果存储目录，目录不存在则自动构建
        :param clean_result_prefix:  string or None
                                   string: 是否对所有行处理功能(不包括清洗功能、创建过些功能)过虑清洗掉的数据
                                   None: 不过虑清洗的数据
        """

        # 1.校验输出目录
        if statis_output_path is not None:
            FileTool.make_dir(statis_output_path)
        run_params_dict = dict()
        run_params_dict['userCleanFilter'] = clean_result_prefix

        # 2.构建task
        task_builder = TaskBuilder()
        task_merge_object, task_other_list, task_meta = task_builder.build(config_flow, config_data_path, run_params_dict)
        self.task_meta = task_meta
        # 3.加载数据
        df_input = input_source
        # df_input = self.load_data(input_source, merge_task_object=task_merge_object, suffix='.xlsx',
        #                           task_meta=self.task_meta)

        # 4.运行任务、处理数据
        df_result = self.run_process(df_input, task_other_list, task_meta, statis_output_path)
        # todo 直接返回一个dataframe
        return df_result
        # 5.保存结果数据
        # 当只有统计类结果时，不需要再保存原始数据，因此没有数据更改
        # self.save_result(df_result, task_meta, output_source)
        # return True

    def load_data(self, input_path, merge_task_object, suffix, task_meta):
        """
        加载要处理的数据
        :param input_path: object
        :param merge_task_object: object，数据合并功能类
        :param suffix: string，文件后缀
        :param task_meta: dict，文件后缀
        :return:
        """
        raise NotImplemented

    def run_process(self, df_input, task_param_list, task_meta, static_output_path, params=None):
        """
        核心运行代码
        :param df_input: DataFrame,待处理数据
        :param task_param_list: list(TaskParam),任务列表
        :param task_meta: dict, 任务元数据
        :param static_output_path: string,统计类Task结果输出目录
        :param params: dict,其他运行参数
        :return:
        """
        raise NotImplemented

    def save_result(self, df_result, task_meta, output_path):
        """
        保存处理结果
        :param df_result: object，数据处理结果
        :param task_meta: dict，元数据
        :param output_path: string，结果输出文件路径/数据库表名
        :return:
        """
        raise NotImplemented

    def process_df_task(self, df_input, task_param_object, statis_out_path):
        """
        处理一些基于DataFrame的处理功能
        :param df_input: DataFrame，处理数据
        :param task_param_object: TaskParam，任务参数
        :param statis_out_path: string，统计结果输出目录
        :return: DataFrame，处理后的结果
        """
        if isinstance(task_param_object.processor, DfProcessInterface):
            # 基于DataFrame处理数据，单独处理
            df_input = task_param_object.processor.process(df_input)
        elif isinstance(task_param_object.processor, DfStatisInterface):
            # 统计功能，不改变原数据
            # todo 现有统计功能可能会修改原数据，因此要复制一份，这里得改！！！！！！！
            df_statis_input = df_input.copy(deep=True)
            df_statis_result = task_param_object.processor.process(df_statis_input)
            del df_statis_input

            s_output_file = task_param_object.output_file
            # 有分隔符代表可能是全路径，没有的只是文件名
            if statis_out_path is not None:
                if os.sep not in s_output_file:
                    s_output_file = os.path.join(statis_out_path, s_output_file)
                if isinstance(df_statis_result, dict):
                    PandaTool.save_many(s_output_file, df_statis_result, index=True)
                else:
                    PandaTool.save(s_output_file, df_statis_result, max_line=self.max_lines, index=True)
        return df_input

def gl_process_line_fun(task_param_object, line_data, task_meta):
    """
    全局通用处理函数，处理一行数据
    :param task_param_object:
    :param line_data: dict, 一行数据
    :return:
    """
    # 1.filter 检查
    if task_param_object.filters:
        for filter_object in task_param_object.filters:
            if filter_object.process((line_data, task_meta)):
                return task_param_object.processor.get_default_value()

    # 2.筛选需要的数据
    # TODO 不指定输入列时返回全部
    if task_param_object.input_column is None or len(task_param_object.input_column) <= 0:
        sel_line_data = line_data
    elif len(task_param_object.input_column) == 1:
        # TODO 只有一个字段时，返回非集合数据
        sel_line_data = line_data[task_param_object.input_column[0]]
    else:
        sel_line_data = {key: line_data[key] for key in task_param_object.input_column}

    # 3.处理数据
    result = task_param_object.processor.process(sel_line_data)

    return result


def gl_clean_line_dict(line_dict):
    """
    主要是清理空数据
    :param line_dict:
    :return:
    """
    line_dict = {key: line_dict[key] if not pd.isna(line_dict[key]) else None for key in line_dict}

    return line_dict


# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 
# author : 
# create_time : 2020/8/21 15:11
# update_time : 2020/8/21 15:11
# copyright : Lavector
# ----------------------------------------------
import time
import datetime


class TimeTool(object):
    @staticmethod
    def get_current_day():
        current_day = time.strftime('%Y%m%d', time.localtime(time.time()))
        return current_day

    @staticmethod
    def get_current_time():
        current_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
        return current_time

    @staticmethod
    def get_current_dgraph_time():
        current_time = time.strftime('%Y-%m-%dT%H:%M:%S', time.localtime(time.time()))
        return current_time

    @staticmethod
    def epoch_seconds(start_datetime, end_datetime):
        """

        :param start_datetime: datetime
        :param end_datetime: datetime
        :return: 返回时间间隔，单位为秒
        """
        td = end_datetime - start_datetime  # datetime.timedelta
        return td.days * 86400 + td.seconds + (float(td.microseconds) / 1000000)

    @staticmethod
    def epoch_hours(start_datetime, end_datetime):
        """

        :param start_datetime: datetime
        :param end_datetime: datetime
        :return: 返回时间间隔，单位为小时
        """
        total_seconds = TimeTool.epoch_seconds(start_datetime, end_datetime)
        hours = int(round(total_seconds / 3600))
        return hours

    @staticmethod
    def epoch_seconds(start_datetime, end_datetime):
        """

        :param start_datetime: datetime
        :param end_datetime: datetime
        :return: 返回时间间隔，单位为秒
        """
        td = end_datetime - start_datetime  # datetime.timedelta
        return td.days * 86400 + td.seconds + (float(td.microseconds) / 1000000)

    @staticmethod
    def string_to_datetime(datetime_string):
        return datetime.datetime.strptime(datetime_string, u"%Y-%m-%d %H:%M:%S")

    @staticmethod
    def datetime_to_string(datetime_data):
        return datetime_data.strftime("%Y-%m-%d %H:%M:%S")


# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 
# author : 
# create_time : 2020/7/1 16:13
# update_time : 2020/7/1 16:13
# copyright : Lavector
# ----------------------------------------------
import sys
import pickle
import pandas as pd
import contextlib
from lavda.analysis.da_tag.tree_like import au_get_tree, au_get_line


@contextlib.contextmanager
def pickle_context(filepath,obj,mode='wb'):
    pass



class _MappingCollect:

    def __init__(self, path):
        """
        mapping表的集合,相当于整个标签词表数据集
        :param path:
        """

        df_sheet = pd.read_excel(path, None)
        sheets = list(df_sheet.keys())
        self.__length = len(sheets)
        self._mapping = [Mapping(path, sheet) for sheet in sheets]
        self.relation = False

    def filter_by(self,name):
        """用于查找某一个sheet对象"""
        pass



class MappingCollect(_MappingCollect):
    def filter_by(self, **kwargs):
        """用于查找某一个sheet对象"""
        name = kwargs.get("name")
        filter_mappings = list(filter(lambda mapping:mapping.sheetname==name,self._mapping))
        if len(filter_mappings) > 0:
            filter_mapping = filter_mappings[0]
        else:
            filter_mapping = None
        return filter_mapping

    def add_sen_sheet(self,sheet_list):
        sheet_list = format_max_xml_kwargs(sheet_list)
        if sheet_list:
            for sheet_name in sheet_list:
                sen_result = self.filter_by(name=sheet_name)
                if sen_result is None:
                    print("情感预设失败,标签词表不含{}sheet".format(sheet_name))
                else:
                    sen_result.sentment =True

    def add_length_sheet(self, sheet_dic):
        sheet_dic = format_max_xml_kwargs(sheet_dic)
        if sheet_dic:
            for sheet_name, tag_length in sheet_dic.items():
                sen_result = self.filter_by(name=sheet_name)
                if sen_result is None:
                    print("距离设置失败,标签词表不含{}sheet".format(sheet_name))
                else:
                    sen_result.length = tag_length

    def __getitem__(self, item):
        return self._mapping[item]

    def to_pickle(self, param):
        save_pickle = open(param,'wb')
        pickle.dump(self,save_pickle)
        save_pickle.close()



class _Mapping:
    """
    对单个mapping表的数据结构进行封装,相当于单一sheet
    """
    def __init__(self, path, sheet):
        tree_line_tup = self.with_draw(path,sheet)
        self.mapping_list = tree_line_tup[1]
        self.mapping_df = tree_line_tup[0]
        self.sheetname = tree_line_tup[-1]
        self.sentment = True
        self.length = 0
        self.relation = None

    def with_draw(self,path,sheet):
        if "-" in sheet:
            sheet_name = sheet.split('-')[0]
            sheet_level = int(sheet.split('-')[-1])
            tree_tup = au_get_tree(path, sheet, sheet_level, sheet_name)
            return tree_tup
        else:
            line_tup = au_get_line(path, sheet)
            return line_tup

    @property
    def mapping_values(self):
        mapping_df = self.mapping_df.astype(str)
        maping_values = mapping_df[self.mapping_list].values
        return maping_values

    @property
    def choice(self):
        return 'content'



class Mapping(_Mapping):

    def __init__(self,path,sheet):
        super(Mapping, self).__init__(path,sheet)
        self.verifi = self.verification()


    def verification(self):
        datapram = {
            'code': 0,
            'msg': "词表正常",
            'sheetname': self.sheetname
        }
        before_verifi_list = self.mapping_values
        verifi_list = [verifi[-1] for verifi in before_verifi_list]
        for veri in verifi_list:
            if str(veri) in ["","||"]:
                datapram['code'] = -1
                datapram['msg'] = "词表异常"
                # print("异常码:{},异常提示:{},词表名:{}".format(datapram['code'],datapram['msg'],datapram['sheetname']))
                return datapram
            if str(veri)[0] == "|" or str(veri[-1]) == "|":
                datapram['code'] = -2
                datapram['msg'] = "词表异常"
                # print("异常码:{},异常提示:{},词表名:{}".format(datapram['code'], datapram['msg'], datapram['sheetname']))


        return datapram




class _Relation_Mapping_collect(MappingCollect):
    def __init__(self,path):
        super(_Relation_Mapping_collect, self).__init__(path)
        self._mapping = [self.add_primary_id(relation_mapping) for relation_mapping in self._mapping]
        self.relation = True

    def add_primary_id(self, mapping):
        name = mapping.sheetname
        df = mapping.mapping_df
        df['{}-id'.format(name)] = range(df.shape[0])
        mapping.mapping_df = df
        return mapping



class Relation_Mapping_collect(_Relation_Mapping_collect):
    def get_map_name_collect(self):
        map_name_collect = [_name.sheetname for _name in self]
        return map_name_collect



def format_max_xml_kwargs(value):
    value = eval(value) if value else value
    return value



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 
# author : 
# create_time : 2020/7/2 14:34
# update_time : 2020/7/2 14:34
# copyright : Lavector
# ----------------------------------------------


class RowFilterInterface(object):
    def process(self, data, **params):
        """
        高层接口
        行数据过虑接口，配置行数据处理功能
        :param data:list(line, task_meta)
                line: dict, 当前行数据
                task_meta: dict, 运行任务的元数据
        :param params: dict, 输入参数
        :return: bool
        """
        pass



# -*- coding:utf-8 -*-
from datetime import datetime


from lavda.util.time_util import TimeTool


from lavda.backexcutor.my_base_executor import *
from lavda.backexcutor.task import *



class DfExecutor(BaseExecutor):
    """
    基于DataFrame的数据处理框架
    """

    def __init__(self, max_lines=700000, jobs=1, mode=0, job_batch_size=1000, channel_col="channel"):
        """

        :param max_lines:int 单个文件保存的最大数据量（行数）
        :param jobs:int 并行数量
        :param mode:int 并行工作模式 ，jobs大于1时有作用 多线程貌似没啥用
                    0：joblib 多进程
                    1：joblib 多线程
                    3：threading 多线程
        :param job_batch_size:int mode=0时，单个job单批处理的最大数据量
        :param channel_col:string channel字段名
        """
        super(DfExecutor, self).__init__(channel_col, max_lines)

        self.max_lines = max_lines
        self.jobs = jobs
        self.mode = mode
        self.job_batch_size = job_batch_size

    def run_process(self, df_input, task_param_list, task_meta, statis_output_path, params=None):
        """
        核心运行代码
        :param df_input: DataFrame，待处理数据
        :param task_param_list: list(TaskParam)，任务列表
        :param task_meta:
        :param statis_output_path: string，统计类Task结果输出目录
        :param params: dict，运行参数
        :return:
        """
        task_time_list = list()
        for task_param_object in task_param_list:
            # todo 统计各任务执行时间
            begin_time = datetime.now()
            # 行接口可以并行处理
            if isinstance(task_param_object.processor, RowInterface):
                # todo 校验输入，输入字段不存在则不执行此任务
                skip = False
                if task_param_object.input_column:
                    cur_column = set(PandaTool.get_column(df_input))
                    for col in task_param_object.input_column:
                        if col not in cur_column:
                            skip = True
                            break
                if skip:
                    continue
                # 处理行功能
                df_input = self.run_row_task(df_input, task_param_object, task_meta)
                # todo 清洗类规则处理完后直接把清洗掉
                if task_param_object.output_column:
                    for clean_column in task_param_object.output_column:
                        if 'tag_clean' in clean_column:
                            df_input = df_input[df_input[clean_column] == 0]
                            df_input.drop(columns=[clean_column], inplace=True)
            else:
                df_input = self.process_df_task(df_input, task_param_object, statis_output_path)
                # todo 清洗类规则处理完后直接把清洗掉
                if task_param_object.output_column:
                    for clean_column in task_param_object.output_column:
                        if 'tag_clean' in clean_column:
                            df_input = df_input[df_input[clean_column] == 0]
                            df_input.drop(columns=[clean_column], inplace=True)

            end_time = datetime.now()
            run_time = TimeTool.epoch_seconds(begin_time, end_time)
            one_task_time = dict()
            one_task_time['start'] = begin_time.strftime('%Y-%m-%d %H:%M:%S')
            one_task_time['end'] = end_time.strftime('%Y-%m-%d %H:%M:%S')
            one_task_time['runTime'] = round(float(run_time), 3)
            one_task_time['class'] = task_param_object.name
            one_task_time['inputColumn'] = task_param_object.input_column

            task_time_list.append(one_task_time)

        task_meta['taskTime'] = task_param_list
        return df_input

    def count_data_meta(self, df_input):
        """
        统计输入数据相关内容
        :param df_input:
        :return:
        """
        data_meta = dict()
        data_meta['count'] = df_input.shape[0]
        data_meta['fieldCount'] = df_input.shape[1]

        return data_meta

    # def load_data(self, data_source, merge_task_object, suffix, task_meta):
    #     """
    #     数据源
    #         如果是数据库，就是一个表名，类型是字符串，
    #         如果是文件，就是一个文件夹，类型是字符串，获取文件夹下所有文件作为数据源
    #         {'type': 'db','name':'table'}
    #         {'type': 'file','name':'folder'}
    #     :param data_source:
    #     :param merge_task_object:
    #     :param suffix:
    #     :param task_meta:
    #     :return:
    #     """
    #     # TODO 原来这里写的是从文件里面加载数据，添加个从数据库里家在数据
    #     # TODO 文件夹返回的dataframe，数据库加载数据返回的是字典，看要不要都改了
    #     data_source_type = data_source['type']
    #     input_path = data_source['name']
    #     if data_source_type == 'db':
    #         df_input = DaDataLoader.load_from_db(input_path, merge_task_object)
    #     else:
    #         df_input = DaDataLoader.load_from_file(input_path, merge_task_object, suffix)
    #         task_meta['data'] = self.count_data_meta(df_input)
    #
    #     return df_input

    # def save_result(self, df_result, task_meta, output_source):
    #     # TODO 原来这里写的是往文件里面写数据，添加个往数据库里写数据
    #     # todo 这里可以就把结果放在表里，把写入文件单独列个功能
    #     print("***********************************************************")
    #     print("***********************************************************")
    #     print("***********************************************************")
    #     print("***********************************************************")
    #     print("***********************************************************")
    #     print('df_result', df_result)
    #     if output_source is not None:
    #
    #         data_source_type = output_source['type']
    #         output_path = output_source['name']
    #         if data_source_type == 'file':
    #             # 返回存储结果的状态
    #             status = DaDataSave.save_to_file(output_path, df_result, task_meta)
    #             return status
    #         elif data_source_type == 'db':
    #             status = DaDataSave.save_to_db(output_path, df_result)
    #             return status

    def run_row_task(self, df_input, task_param_object, task_meta):
        # print('df_input',df_input)
        # print('task_meta',task_meta)
        # print('task_param_object',task_param_object)
        # 实际执行处理任务
        if self.jobs > 1:
            all_line_result = self._parallel_row_process(df_input, task_param_object, task_meta, self.jobs)
        else:
            all_line_result = self._single_row_process(df_input, task_param_object, task_meta)

        # 处理结果添加到原有数据集上
        output_column = task_param_object.output_column
        if len(output_column) == 1:
            df_input[output_column[0]] = all_line_result
        else:
            for i, one_column in enumerate(output_column):
                one_column_result = [item[i] for item in all_line_result]
                df_input[one_column] = one_column_result
        return df_input

    def _single_row_process(self, df_input, task_param_object, task_meta):
        """
        串行处理数据
        :param df_input: DataFrame，待处理数据
        :param task_param_object: list(TaskParam)，任务列表
        :param task_meta: dict,任务元数据
        :return:
        """
        # 将数据转换为列表
        # print('df_input', df_input)
        data_list = gl_format_df_data(df_input)
        all_line_result = [gl_process_line_fun(task_param_object, line_data, task_meta) for line_data in data_list]

        return all_line_result

    def _parallel_row_process(self, df_input, task_param_object, task_meta, jobs):
        """
        并行数据处理，感觉没必要
        :param df_input: DataFrame，待处理数据
        :param task_param_object:list(TaskParam)，任务列表
        :param jobs: int，并发数量
        :return:
        """
        # 将数据转换为列表
        data_list = gl_format_df_data(df_input)
        return


def gl_format_df_data(df_input):
    """
    格式化dataframe 数据为python list
    因为DataFrame的遍历操作不如list快
    :param df_input:
    :return:
    """
    data_list = df_input.to_dict(orient="records")
    data_list = [gl_clean_line_dict(item) for item in data_list]

    return data_list



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 
# author : 
# create_time : 2020/7/1 16:20
# update_time : 2020/7/1 16:20
# copyright : Lavector
# ----------------------------------------------
import os
import pandas as pd
import xlrd


class LabelExcelMerge(object):
    def process(self, input_file_path, sheet_name=None):
        workbook = xlrd.open_workbook(input_file_path)

        # 获取所有sheet
        # print('打印所有sheet:', workbook.sheet_names())
        if not sheet_name:
            target_sheet = workbook.sheet_by_index(0)  # sheet索引从0开始
        else:
            target_sheet = workbook.sheet_by_name(sheet_name)
        rows_num = target_sheet.nrows
        cols_num = target_sheet.ncols

        # print(target_sheet.merged_cells)
        total_data_list = list()
        for r in range(rows_num):
            line_data_list = list()
            for c in range(cols_num):
                cell_value = target_sheet.row_values(r)[c]
                # print('第%d行第%d列的值：[%s]' % (r, c, sheet2.row_values(r)[c]))
                if cell_value is None or cell_value == '':
                    cell_value = (self.get_merged_cells_value(target_sheet, r, c))
                line_data_list.append(cell_value)

            total_data_list.append(line_data_list)

        return total_data_list

    def get_merged_cells(self, sheet):
        """
        获取所有的合并单元格，格式如下：
        [(4, 5, 2, 4), (5, 6, 2, 4), (1, 4, 3, 4)]
        (4, 5, 2, 4) 的含义为：行 从下标4开始，到下标5（不包含）  列 从下标2开始，到下标4（不包含），为合并单元格
        :param sheet:
        :return:
        """
        return sheet.merged_cells

    def get_merged_cells_value(self, sheet, row_index, col_index):
        """
        先判断给定的单元格，是否属于合并单元格；
        如果是合并单元格，就返回合并单元格的内容
        :return:
        """
        merged = self.get_merged_cells(sheet)
        for rlow, rhigh, clow, chigh in merged:
            if row_index >= rlow and row_index < rhigh:
                if col_index >= clow and col_index < chigh:
                    cell_value = sheet.cell_value(rlow, clow)
                    # print('该单元格[%d,%d]属于合并单元格，值为[%s]' % (row_index, col_index, cell_value))
                    return cell_value
                    break
        return None


class LabelDictInterface(object):
    def set_data_column(self, data_column):
        """

        :param data_column: list   标签所对应的数据列名
        :return:
        """
        pass

    def get_data_column(self):
        pass

    def get_label_name(self):
        """
        标签名称
        :return:
        """
        pass

    def set_label_name(self, label_name):
        pass

    def get_label_id_dict(self):
        pass

    def get_item_by_id(self, label_id):
        pass

    def get_label_text_by_id(self, label_id):
        pass

    def get_last_labels_item(self):
        """
        获取最后一级label 信息,即实际用于打标签的内容
        :return:
        """
        pass

    def get_parent_label_text(self, label_id, include_me=True):
        pass

    def get_label_detail_column(self, include_label_id=True):
        """
        标签详细信息列名
        :param include_label_id:  是否包含最后一级label id
        :return:
        """

    def get_label_detail(self, label_id, include_label_id=True):
        """
        标签详细信息
        :param label_id:
        :param include_label_id:
        :return:
        """
        pass

    def get_column_prefix(self):
        """
        列名前缀，用于作为统计汇总使用
        :return:
        """
        return "la_"

    def print_all(self):
        pass

    def print_detail(self):
        pass

    def get_processor(self, processor):
        pass

    def set_processor(self):
        pass


class TreeLabelDict(LabelDictInterface):
    def __init__(self, label_id_dict, level_count, label_name=None):
        self.label_id_dict = label_id_dict
        self.level_count = level_count
        self.label_name = label_name

        # TODO 用于打标签的数据列名,暂时放在这里吧
        self.data_column = None
        # TODO 标签处理类，暂时放这里吧
        self.processor = None

    def set_processor(self, processor):
        self.processor = processor

    def get_processor(self):
        return self.processor

    def set_data_column(self, data_column):
        self.data_column = data_column

    def get_data_column(self):
        return self.data_column

    def get_label_name(self):
        return self.label_name

    def set_label_name(self, label_name):
        self.label_name = label_name

    def get_label_id_dict(self):
        return self.label_id_dict

    def get_level_count(self):
        return self.level_count

    def get_item_by_id(self, label_id):
        return self.label_id_dict[label_id]

    def get_label_text_by_id(self, label_id):
        return self.label_id_dict[label_id]['text']

    def get_parent_by_level(self, current_label_id, parent_level):
        """
        查找相应级别的父类标签
        :param current_label_id:
        :param parent_level:
        :return:
        """
        parent_id = self.label_id_dict[current_label_id]['parent']
        while parent_id >= 0:
            parent_item = self.label_id_dict[parent_id]
            if parent_item['level'] == parent_level:
                return parent_item
            parent_id = parent_item['parent']

        return None

    def get_last_labels_item(self):
        """
        获取最后一级label 信息,即实际用于打标签的内容
        :return:
        """
        label_item_list = list()
        for label_id in self.label_id_dict:
            label_item = self.label_id_dict[label_id]
            if label_item['level'] == self.level_count:
                label_item_list.append(label_item)

        return label_item_list

    def get_label_detail_column(self, include_label_id=True):
        column_list = list()
        label_level = self.get_level_count()
        for i in range(label_level):
            column_list.append(self.get_column_prefix() + self.label_name + "_" + str(i + 1))
        if include_label_id:
            column_list.append(self.get_column_prefix() + self.label_name + "_" + "id")

        return column_list

    def get_label_detail(self, label_id, include_label_id=True):
        detail_list = self.get_all_parent_label_text(label_id)
        if include_label_id:
            detail_list.append(label_id)

        return detail_list

    def get_all_parent_label_text(self, current_label_id, include_me=True):
        parent_list = list()
        parent_id = self.label_id_dict[current_label_id]['parent']
        while parent_id >= 0:
            parent_list.insert(0, self.label_id_dict[parent_id]['text'])
            parent_id = self.label_id_dict[parent_id]['parent']
        if include_me:
            parent_list.append(self.get_item_by_id(current_label_id)['text'])

        return parent_list

    def print_all(self):
        level_count_dict = {level + 1: 0 for level in range(self.level_count)}
        for label_id in self.label_id_dict:
            level_count_dict[self.label_id_dict[label_id]['level']] += 1
        print(level_count_dict)

    def print_detail(self):
        for label_id in self.label_id_dict:
            print(self.label_id_dict[label_id])

    def get_dict(self):
        return self.label_id_dict


class PandaLabelDictTool(object):
    def check_file(self, input_file_path, file_suffix):
        parent_file_path, temp_file_name = os.path.split(input_file_path)
        file_name, extension = os.path.splitext(temp_file_name)
        extension = extension.lower()
        extension = extension.replace(".", "")
        if extension in file_suffix:
            return True
        else:
            raise Exception("unsupport file extension = " + extension)

    def trans(self, total_data_list):
        """
        二维数据置换
        :param total_data_list:
        :return:
        """
        old_max_rows = len(total_data_list)
        old_max_column = 0
        for line in total_data_list:
            current_column = len(line)
            if current_column > old_max_column:
                old_max_column = current_column

        result_data_list = list()
        for i in range(old_max_column):
            sub_data_list = list()
            for j in range(old_max_rows):
                if i < len(total_data_list[j]):
                    sub_data_list.append(total_data_list[j][i])
                else:
                    # TODO 空值填充
                    sub_data_list.append(None)
            result_data_list.append(sub_data_list)

        return result_data_list

    def is_none_value(self, cell_value):
        if pd.isna(cell_value):
            return True
        tmp_cell_value = str(cell_value)
        if not tmp_cell_value:
            return True

        return False

    def drop_none_row(self, total_data_list):
        """
        删除全为None 或空值的行
        :param total_data_list:
        :return:
        """
        result_data_list = list()
        for line in total_data_list:
            is_delete = True
            for cell_value in line:
                if not self.is_none_value(cell_value):
                    is_delete = False
                    break
            if not is_delete:
                result_data_list.append(line)

        return result_data_list

    def drop_none_column(self, total_data_list):
        """
        删除全为None 或空值的行
        :param total_data_list:
        :return:
        """
        delete_column_index = set()
        column_data_list = self.trans(total_data_list)
        for i, line in enumerate(column_data_list):
            is_delete = True
            for cell_value in line:
                if not self.is_none_value(cell_value):
                    is_delete = False
                    break
            if is_delete:
                delete_column_index.add(i)

        result_data_list = list()
        for line in total_data_list:
            sub_list = list()
            for j, cell_value in enumerate(line):
                if j not in delete_column_index:
                    sub_list.append(cell_value)
            result_data_list.append(sub_list)

        return result_data_list

    def check_has_non_value(self, total_data_list):
        for i, line in enumerate(total_data_list):
            for j, cell_value in enumerate(line):
                if self.is_none_value(cell_value):
                    raise Exception("has non value" + ", line=" + str(i) + ", col=" + str(j))

    def drop_end_non_value(self, total_data_list, total_label_level):
        """
        除最后一级标签外， 删除末尾的连续 空值
        :param total_data_list:
        :return:
        """
        result_data = list()
        for i, line in enumerate(total_data_list):
            label_level = i + 1
            if label_level < total_label_level:
                del_list = list()
                data_index = len(line) - 1
                while data_index >= 0:
                    item = line[data_index]
                    if self.is_none_value(item):
                        del_list.append(data_index)
                    else:
                        break
                    data_index -= 1

                if del_list:
                    sub_list = line[0: del_list[len(del_list) - 1]]
                else:
                    sub_list = line
            else:
                sub_list = line
            result_data.append(sub_list)

        return result_data

    def load_tree_dict(self, input_file_path,
                       sheet_name=None,
                       tree_format=0,
                       header=False,
                       total_label_level=5,
                       label_name=None,
                       split_text=False):
        """

        :param input_file_path: string, 输入文件路径
        :param sheet_name: string, sheet name
        :param tree_format: int, 格式，0：竖形树， 1：横着的树
        :param header: bool, 是否有表头，默认不保留表头信息
        :param total_label_level: int, 标签词典级别
        :param label_name: string, 标签词典级别
        :param split_text: bool, 最后一级是否需要拆分文本
        :return:
        """
        # 1.校验输入文件
        file_suffix = ['xlsx', 'xls']
        self.check_file(input_file_path, file_suffix)

        # 2.处理合并单元格
        excel_merge = LabelExcelMerge()
        dict_array = excel_merge.process(input_file_path, sheet_name=sheet_name)

        # 检查是否有表头
        # TODO 默认不需要表头数据
        if header:
            # 需要去掉表头数据
            dict_array = dict_array[1:]

        # 3.横排的要做转置
        if tree_format == 1:
            # 矩阵转换，横排的要做转换
            dict_array = self.trans(dict_array)

        # 4. 清理
        # 除最后一级外，删除所有行末尾的空值
        dict_array = self.drop_end_non_value(dict_array, total_label_level)
        dict_array = self.drop_none_row(dict_array)
        dict_array = self.drop_none_column(dict_array)

        # 除最后一级外，没能有空值
        if total_label_level > 1:
            self.check_has_non_value(dict_array[0: total_label_level - 1])

        # 以第一行为标准确定完整的列数
        max_column_count = len(dict_array[0])

        # 6. 加载词典信息
        # 全局信息
        cell_pos_dict = dict()
        # 记录最后一级以外的父节点信息
        cell_pos_list = list()

        label_id_dict = dict()
        total_label_id_count = 0

        for i, line in enumerate(dict_array):
            current_level = i + 1
            if current_level > total_label_level:
                current_level = total_label_level
            parent_level = current_level - 1

            sub_pos_list = list()
            cell_pos_list.append(sub_pos_list)
            for j, cell_value in enumerate(line):
                # 超过最大列的不处理
                if j >= max_column_count:
                    continue

                if pd.isna(cell_value):
                    sub_pos_list.append(None)
                    continue
                cell_value = str(cell_value).strip()
                if not cell_value:
                    sub_pos_list.append(None)
                    continue

                if current_level == 1:
                    parent_label_id = -1
                    parent_label_item = None
                else:
                    parent_label_item = cell_pos_list[parent_level - 1][j]
                    if parent_label_item is None:
                        raise Exception("parent is None: line=" + str(i) + ", row" + str(j))
                    parent_label_id = parent_label_item['id']

                # TODO 只对最后一级作拆分
                if current_level == total_label_level and split_text:
                    sub_cell_value_list = cell_value.split("|")
                    sub_cell_value_list = [sub_value.strip() for sub_value in sub_cell_value_list if sub_value.strip()]

                    for sub_cell_value in sub_cell_value_list:
                        pos_value = str(parent_label_id) + "-" + str(current_level) + "-" + sub_cell_value
                        if pos_value in cell_pos_dict:
                            sub_pos_list.append(cell_pos_dict[pos_value])
                            continue

                        new_label_item = dict()
                        new_label_item['id'] = total_label_id_count
                        new_label_item['level'] = current_level
                        new_label_item['parent'] = parent_label_id
                        new_label_item['text'] = sub_cell_value
                        cell_pos_dict[pos_value] = new_label_item
                        sub_pos_list.append(new_label_item)

                        label_id_dict[new_label_item['id']] = new_label_item

                        total_label_id_count += 1

                else:
                    pos_value = str(parent_label_id) + "-" + str(current_level) + "-" + cell_value
                    if pos_value in cell_pos_dict:
                        sub_pos_list.append(cell_pos_dict[pos_value])
                        continue

                    new_label_item = dict()
                    new_label_item['id'] = total_label_id_count
                    new_label_item['level'] = current_level
                    new_label_item['parent'] = parent_label_id
                    new_label_item['text'] = cell_value
                    cell_pos_dict[pos_value] = new_label_item
                    sub_pos_list.append(new_label_item)
                    label_id_dict[new_label_item['id']] = new_label_item

                    total_label_id_count += 1

        label_dict_object = TreeLabelDict(label_id_dict, total_label_level, label_name)

        return label_dict_object



# -*- coding: utf-8 -*-
import re
import os


# from lavda.analysis.da_clean.df_distinct import *
# from lavda.analysis.da_filter.clean_filter import *
# from lavda.analysis.da_filter.column_match_filter import *
# from lavda.analysis.da_pre.create_data_index import *
#
# from lavda.backexcutor.analysis_interface import *
# from lavda.backexcutor.py_model_loader import *
from lavda.analysis.da_clean.clean_interface import CleanInterface
from lavda.analysis.da_clean.df_distinct import DfDistinct
from lavda.analysis.da_filter.clean_filter import CleanFilter
from lavda.analysis.da_filter.column_match_filter import ColumnNotMatchFilter
from lavda.analysis.da_pre.create_data_index import CreateDataIndex
from lavda.backexcutor.analysis_interface import *
from lavda.backexcutor.py_model_loader import PyModelLoader


class TaskParam(object):
    def __init__(self, input_column, output_column, processor, name=None):
        """

        :param input_column: None or list, 输入字段，适用于行接口
        :param output_column: None or list，输出字段，适用于行接口
        :param processor: object,功能处理类
        :param name:string, 名称
        """
        self._input_column = input_column
        self._output_column = output_column
        self._processor = processor
        if name is not None:
            self._name = name
        else:
            self._name = str(type(processor).__name__)

        # 输出结果文件名/可以是包含完整路径的文件名，适用于统计类接口
        # todo 像统计结果可以直接展示到页面，也可以下载
        self._output_file = None

        # 清洗策略
        self._filters = list()

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def filters(self):
        return self._filters

    @filters.setter
    def filters(self, value):
        self._filters = value

    @property
    def output_file(self):
        return self._output_file

    @output_file.setter
    def output_file(self, value):
        self._output_file = value

    @property
    def input_column(self):
        return self._input_column

    @input_column.setter
    def input_column(self, value):
        self._input_column = value

    @property
    def output_column(self):
        return self._output_column

    @output_column.setter
    def output_column(self, value):
        self._output_column = value

    @property
    def processor(self):
        return self._processor

    @processor.setter
    def processor(self, value):
        self._processor = value


class ConfigLabel(object):
    """
    配置文件标签
    """
    # 类名
    CLASS = 'class'
    # 输入字段,暂时不用
    INPUT_COLUMN = "input_column"
    # 输出字段,暂时不用
    OUTPUT_COLUMN = "output_column"
    # 类初始化参数
    INIT = "init"
    # configs 资源文件
    CONFIG = "configs"
    # pair-name，输入输出字段名
    PAIR_COLUMN = 'pair_column'
    # 输出文件
    OUTPUT_FILE = 'output_file'
    # 过滤
    FILTER = "filter"


'''

'''
a = {'class': '', 'input_column': [], 'output_column': [], 'init': '', 'configs': '', 'output_file': '', 'filter': ''}
test = {'distinct': {'columns': ['content', 'title', 'datetime'], 'configs': {}},
        'mark': {'columns': ['content', 'title'], 'configs': {'topic': True}}}


class TaskBuilder(object):
    def _parse_common_columns(self, col_str, default_value=None):
        # todo 抄的国东的
        if not col_str:
            return default_value
        tmp_list = re.split("[,，]", col_str)
        tmp_list = [item.strip() for item in tmp_list if item.strip()]
        if len(tmp_list) <= 0:
            return default_value

        return tmp_list

    def _build_row_task(self, item, class_object, run_params_dict, default_value=None):
        """
        构建行接口任务类
        :param item:
        :param class_object:
        :param run_params_dict:
        :param default_value:
        :return:
        """

        # 1.构建输入输出列名
        columns_result = list()
        if ConfigLabel.INPUT_COLUMN in item or ConfigLabel.OUTPUT_COLUMN in item:
            input_column = None
            if ConfigLabel.INPUT_COLUMN in item:
                input_column = item[ConfigLabel.INPUT_COLUMN]
            # if not input_column:
            #     raise Exception("input columns has no data")
            output_column = None
            if ConfigLabel.OUTPUT_COLUMN in item:
                output_column = item[ConfigLabel.OUTPUT_COLUMN]
            # if not output_column:
            #     raise Exception("output column is empty for class = " + item[ConfigLabel.CLASS])

            columns_result.append((input_column, output_column))
            return columns_result
        elif ConfigLabel.PAIR_COLUMN in item:  # TODO 这儿不理解
            tmp_list = re.split("[;；]", item[ConfigLabel.PAIR_COLUMN])
            tmp_list = [item.strip() for item in tmp_list if item.strip()]

            for sub_str in tmp_list:
                if "=" not in sub_str:
                    raise Exception("pair column must has '=' for class = " + item[ConfigLabel.CLASS])
                sub_list = sub_str.split("=")
                sub_list = [item.strip() for item in sub_list if item.strip()]

                if 2 < len(sub_list) or len(sub_list) <= 0:
                    raise Exception("invalid pair column =" + item[ConfigLabel.PAIR_COLUMN])

                if len(sub_list) == 2:
                    input_column = self._parse_common_columns(sub_list[0], default_value)
                    output_column = self._parse_common_columns(sub_list[1], default_value)

                    columns_result.append((input_column, output_column))
                else:
                    if sub_str.startswith("="):
                        output_column = self._parse_common_columns(sub_list[0])
                        columns_result.append((default_value, output_column))
                    else:
                        input_column = self._parse_common_columns(sub_list[0])
                        columns_result.append((input_column, default_value))
                # 应该具有相同的输入，输出
                input_set = set()
                output_set = set()
                for tmp_col_item in columns_result:
                    in_len = len(tmp_col_item[0]) if tmp_col_item[0] else 0
                    ou_len = len(tmp_col_item[1]) if tmp_col_item[1] else 0
                    input_set.add(in_len)
                    output_set.add(ou_len)

                if len(input_set) > 1:
                    raise Exception("pair must has same number of input column for class =" + item[ConfigLabel.CLASS])
                if len(output_set) > 1:
                    raise Exception("pair must has same number of output column for class =" + item[ConfigLabel.CLASS])
        else:
            raise Exception("has no input/output column")

        # 2. 构建清洗类
        filter_list = list()
        # 创建索引的功能不需要过滤功能
        if not isinstance(class_object, CreateDataIndex):
            # todo 默认所有非清洗类行接口都要使用此策略
            if not isinstance(class_object, CleanInterface):
                if run_params_dict['userCleanFilter']:
                    filter_list.append(CleanFilter(run_params_dict['userCleanFilter']))
            # todo 这儿不懂
            if ConfigLabel.FILTER in item:
                filter_item = item[ConfigLabel.FILTER]
                for one_key in filter_item:
                    # todo 暂时只定义的一种过滤策略
                    if one_key == "column_filter_not_match":
                        filter_list.append(ColumnNotMatchFilter(filter_item[one_key]))

        # 3. 构建Task
        task_param_list = list()
        for one_result in columns_result:
            one_task_param = TaskParam(one_result[0], one_result[1], class_object)
            one_task_param.filters = filter_list
            task_param_list.append(one_task_param)

        return task_param_list

    # def make_task(self, configs, run_params_dict):
    #     # TODO 直接抄的国东的
    #     if isinstance(configs, list):
    #         config_all = configs
    #     else:
    #         raise Exception('unknown configs data')
    #     if len(config_all) <= 0:
    #         raise Exception('empty task configs')
    #
    #     loader = PyModelLoader()
    #     task_param_list = list()
    #     is_load_object = True
    #     for item in config_all:
    #         # print(item)
    #         # 1. 生成任务类
    #         if isinstance(item[ConfigLabel.CLASS], TaskInterface):
    #             class_object = item[ConfigLabel.CLASS]
    #             is_load_object = False
    #         # todo 这里还有问题
    #         else:
    #             init_dict = item[ConfigLabel.INIT] if ConfigLabel.INIT in item else None
    #             if init_dict:
    #                 class_object = loader.load(item[ConfigLabel.CLASS], **init_dict)
    #             else:
    #                 class_object = loader.load(item[ConfigLabel.CLASS])
    #         # 2. 生成相应的数据字段
    #         default_column = None
    #         if isinstance(class_object, RowInterface):
    #             # 行接口要为解析输入、输出字段
    #             # TODO 这儿
    #             sub_task_list = self._build_row_task(item, class_object, run_params_dict, default_column)
    #             task_param_list.extend(sub_task_list)
    #         elif isinstance(class_object, DfProcessInterface):
    #             # DataFrame 任务生成类接口
    #             # 去重类特殊处理
    #             if isinstance(class_object, DfDistinct):
    #                 distinct_init = item[ConfigLabel.INIT]
    #                 task_param_list.append(
    #                     TaskParam(distinct_init['input_column'], distinct_init['output_column'], class_object))
    #                 print(task_param_list)
    #             else:
    #                 task_param_list.append(TaskParam(default_column, default_column, class_object))
    #         elif isinstance(class_object, DfStatisInterface):
    #             # 统计类接口要有输出结果文件名
    #             if ConfigLabel.OUTPUT_FILE not in item:
    #                 raise Exception("统计类接口需要包含output_file字段")
    #             one_task_param = TaskParam(default_column, default_column, class_object)
    #             one_task_param.output_file = item[ConfigLabel.OUTPUT_FILE]
    #             task_param_list.append(one_task_param)
    #         elif isinstance(class_object, DfMergeInterface):
    #             one_task_param = TaskParam(default_column, default_column, class_object)
    #             task_param_list.append(one_task_param)
    #         else:
    #             raise Exception("unknown interface for class = " + str(type(class_object).__name__))
    #
    #         # 3. 初始化配置资源
    #         # todo 用户自己初始化的功能，暂时不再加载资源了
    #         if is_load_object:
    #             if ConfigLabel.CONFIG in item and item[ConfigLabel.CONFIG]:
    #                 config_dict = item[ConfigLabel.CONFIG]
    #                 class_object.load_config_data(**config_dict)
    #     return task_param_list
    def make_task(self, config, config_data_path, run_params_dict):
        """
        校验任务参数并生成任务
        :param config:
        :return:
        """
        if isinstance(config, list):
            print('这儿没走')
            config_list = config
        else:
            raise Exception("unknown configs data")


        if len(config_list) <= 0:
            raise Exception("empty task configs!")

        loader = PyModelLoader()
        task_param_list = list()
        for item in config_list:
            # 1.生成任务类
            if ConfigLabel.CLASS not in item:
                raise Exception("has not task class ")
            # TODO 是否为框架自动生成的类
            is_load_object = True
            if isinstance(item[ConfigLabel.CLASS], TaskInterface):
                class_object = item[ConfigLabel.CLASS]
                is_load_object = False
            else:
                int_dict = item[ConfigLabel.INIT] if ConfigLabel.INIT in item else None
                if int_dict:
                    class_object = loader.load(item[ConfigLabel.CLASS], **int_dict)
                else:
                    class_object = loader.load(item[ConfigLabel.CLASS])
            # 2.生成相应的数据字段
            default_column = None
            if isinstance(class_object, RowInterface):
                # 行接口为要解析输入、输出字段
                sub_task_list = self._build_row_task(item, class_object, run_params_dict, default_column)
                task_param_list.extend(sub_task_list)
            elif isinstance(class_object, DfProcessInterface):
                # DataFrame 任务生成类接口
                # TODO 去重类特殊处理  <lavda.analysis.da_clean.df_distinct.DfDistinct object at 0x00000249D59B0610>
                # TODO 去重类实在是太特殊了，因此这里只能使用特例了
                if isinstance(class_object, DfDistinct):
                    distinct_init = item[ConfigLabel.INIT]
                    task_param_list.append(TaskParam([distinct_init['input_column']], [distinct_init['output_column']], class_object))
                else:
                    task_param_list.append(TaskParam(default_column, default_column, class_object))
            elif isinstance(class_object, DfStatisInterface):
                # 统计类接口要有输出结果文件名
                if ConfigLabel.OUTPUT_FILE not in item:
                    raise Exception("统计类接口需要包含output_file字段")
                one_task_param = TaskParam(default_column, default_column, class_object)
                one_task_param.output_file = item[ConfigLabel.OUTPUT_FILE]

                task_param_list.append(one_task_param)
            elif isinstance(class_object, DfMergeInterface):
                one_task_param = TaskParam(default_column, default_column, class_object)
                task_param_list.append(one_task_param)
            else:
                raise Exception("unknown interface for class = " + str(type(class_object).__name__))

            # 3.初始化配置资源
            # TODO 用户自己初始化的功能，暂时不再加载资源了
            if is_load_object:
                if ConfigLabel.CONFIG in item:
                    config_dict = item[ConfigLabel.CONFIG]
                    if config_dict:
                    # todo 这里改了，可能有问题
                    # 校验文件
                        for one_key in config_dict:
                            one_value = config_dict[one_key]
                            if not os.path.isfile(one_key):
                                one_value = os.path.join(config_data_path, one_value)
                                config_dict[one_key] = one_value
                            # if not os.path.exists(one_value):
                            #     raise Exception("configs file not exist = " + one_value)
                            class_object.load_config_data(**config_dict)

        return task_param_list

    def split_task(self, task_param_list):
        """
        拆分task，分为一般task和数据合并task
        TODO 直接抄的国东的
        :param task_param_list:
        :return:
        """
        if isinstance(task_param_list[0].processor, DfMergeInterface):
            task_merge = task_param_list[0]
            if len(task_param_list) > 1:
                task_other = task_param_list[1:]
            else:
                task_other = list()
        else:
            task_merge = None
            task_other = task_param_list

        # TODO 校验任务，只有特定接口的功能类可以作为任务流使用
        for task_param in task_other:
            if isinstance(task_param.processor, TaskInterface):
                continue
            raise Exception("unsupport task type for" + str(type(task_param.processor).__name__))

        return [task_merge, task_other]

    def make_task_meta(self, task_param_list):
        """
        构建一下运行时的任务数据，
        :param task_param_list:
        :return:
        """
        meta_dict = dict()
        # 获取所有清洗数据的输出字段
        meta_dict['cleanOutput'] = list()
        for param_object in task_param_list:
            if isinstance(param_object.processor, CleanInterface):
                if param_object.output_column:
                    meta_dict['cleanOutput'].extend(param_object.output_column)

        # 获取所有任务数量
        meta_dict['taskCount'] = len(task_param_list)
        row_task_count = 0
        df_process_count = 0
        df_statis_count = 0
        df_merge_count = 0
        for task_param_object in task_param_list:
            if isinstance(task_param_object.processor, DfProcessInterface):
                df_process_count += 1
            elif isinstance(task_param_object.processor, RowInterface):
                row_task_count += 1
            elif isinstance(task_param_object.processor, DfStatisInterface):
                df_statis_count += 1
            elif isinstance(task_param_object.processor, DfMergeInterface):
                df_merge_count += 1

        meta_dict['rowTaskCount'] = row_task_count
        meta_dict['dfProcessTaskCount'] = df_process_count
        meta_dict['dfStatisTaskCount'] = df_statis_count
        meta_dict['dfMergeTaskCount'] = df_merge_count

        return meta_dict

    def build(self, config, config_data_path, run_params_dict):
        task_param_list = self.make_task(config, config_data_path, run_params_dict)
        split_result = self.split_task(task_param_list)
        meta_dict = self.make_task_meta(task_param_list)
        return split_result + [meta_dict]


if __name__ == '__main__':
    if isinstance('DfDistinct', DfDistinct):
        print(1)



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose: 地理信息相关功能
# author : 
# create_time : 2020/6/29 15:06
# update_time : 2020/6/29 15:06
# copyright : Lavector
# ----------------------------------------------
"""
地理信息处理
"""
import numpy as np
from lavda.util.mysql_util import *


class AreaDictInterface(object):
    def get(self):
        pass

    def get_by_area_code(self, area_code):
        pass

    def find(self, text, level=None):
        pass

    def print_all(self):
        pass


class MainAreaDict(AreaDictInterface):
    """
    中国主要城市信息数据
    """

    def __init__(self, area_list):
        self.area_list = area_list

        self.area_dict = dict()
        self.name_code_dict = dict()
        for item in self.area_list:
            area_code = item['areaCode']
            self.area_dict[area_code] = item

            name = item['name']
            short_name = item['shortName']
            if name not in self.name_code_dict:
                self.name_code_dict[name] = set()
            if short_name not in self.name_code_dict:
                self.name_code_dict[short_name] = set()

            self.name_code_dict[name].add(area_code)
            self.name_code_dict[short_name].add(area_code)

    def get(self):
        return self.area_list

    def get_by_area_code(self, area_code):
        if area_code not in self.area_dict:
            return None
        return self.area_dict[area_code]

    def find(self, text, level=None):
        if isinstance(level, int):
            current_level_set = {level}
        elif isinstance(level, (tuple, list)):
            current_level_set = set(level)
        elif isinstance(level, set):
            current_level_set = level
        else:
            current_level_set = None

        result = list()

        if text in self.name_code_dict:
            area_code_set = self.name_code_dict[text]

            for area_code in area_code_set:
                current_area_item = self.area_dict[area_code]
                if current_level_set:
                    if current_area_item['level'] in current_level_set:
                        result.append(current_area_item)
                else:
                    result.append(current_area_item)

        return result

    def print_all(self):
        print("data count = ", len(self.area_list))


class CityRankDict(AreaDictInterface):
    """
    中国主要级别城市排行
    """

    def __init__(self, area_list):
        self.area_list = area_list

        self.area_dict = dict()
        for item in self.area_list:
            self.area_dict[item['areaCode']] = item

    def get(self):
        return self.area_list

    def get_by_area_code(self, area_code):
        if area_code not in self.area_dict:
            return None
        return self.area_dict[area_code]

    def find(self, text, level=None):
        """
        在指定的level内进行匹配查找
        :param text:
        :return: list(dict)
        """
        if isinstance(level, int):
            current_level_set = {level}
        elif isinstance(level, (tuple, list)):
            current_level_set = set(level)
        elif isinstance(level, set):
            current_level_set = level
        else:
            current_level_set = None

        result = list()
        for item in self.area_list:
            if item['name'] == text or item['shortName'] == text:
                if current_level_set and item['level'] not in current_level_set:
                    continue
                result.append(item)

        return result

    def print_all(self):
        print("data count = ", len(self.area_list))


class CityExtractProcessor(object):
    """
    从文本中提取城市信息
    """
    SPE_CITY = {'北京', '天津', '重庆', '上海', '北京市', '上海市', '重庆市', '天津市'}

    def __init__(self, main_area_object, city_list):
        self.main_area_object = main_area_object
        self.city_list = city_list


    def process(self, data, **params):
        """
        从文本中提取城市信息，提取最低级别城市信息
        支持的格式：
        重庆 铜梁县
        北京
        北京 东城区
        湖北 荆州
        湖北 仙桃
        广东
        沈阳 法库
        :param data:
        :param params:
        :return: list(dict)，返回完整的城市信息
        """
        # TODO 2021.8.13 增加了这里的城市判断
        if type(data) == dict:
            city_column_name_list = ['city', 'introduce']
            for column_name in city_column_name_list:
                if column_name not in data:
                    data[column_name] = ''
            data_city = data['city']
            data_city = str(data_city).strip()
            if not data_city:
                city_judgment_text = ''.join([str(data[i]) for i in city_column_name_list if data[i]])
                data_city = self.city_judgment(city_judgment_text)
                if not data_city:
                    return list()
            text = str(data_city).strip()
        else:
            if data is None:
                return list()
            text = str(data).strip()
            if not text:
                return list()

        # TODO
        tmp_list = text.split(" ")
        tmp_list = [value.strip() for value in tmp_list if value.strip()]

        # 1.直接在city rank词典里面查找
        parent_city_str = None
        if len(tmp_list) == 2:
            if tmp_list[0] in self.SPE_CITY:
                parent_city_str = tmp_list[0]
                cur_city_str = tmp_list[1]
            elif tmp_list[1] in self.SPE_CITY:
                parent_city_str = None
                cur_city_str = tmp_list[1]
            else:
                parent_city_str = tmp_list[0]
                cur_city_str = tmp_list[1]
        elif len(tmp_list) == 1:
            cur_city_str = tmp_list[0]
        else:
            # TODO 未知的格式
            return list()

        # TODO 有些行政区域会有调整，如县改为区，因此可能就无法匹配到
        # TODO 生成多个可能的候选城市,把可能性高的放在前面
        can_cur_city_set = {cur_city_str}
        if cur_city_str.endswith("县"):
            can_cur_city_set.add(cur_city_str.replace("县", "区"))

        # 先查找子类
        tmp_target_result = dict()
        for cur_str in can_cur_city_set:
            one_target_result = self.main_area_object.find(cur_str)
            for one_item in one_target_result:
                tmp_target_result[one_item['areaCode']] = one_item

        # 再查找父类
        # 先查找父类，重复性很低的
        if parent_city_str:
            target_parent_result = self.main_area_object.find(parent_city_str, {1, 2})
            if target_parent_result and len(target_parent_result) >= 1:
                # 只使用第一个
                target_parent_result = target_parent_result[0]

                # 筛选子级
                sub_target_result = dict()
                for tmp_area_code in tmp_target_result:
                    tmp_item = tmp_target_result[tmp_area_code]
                    if tmp_item['parent'] == target_parent_result['areaCode']:
                        sub_target_result[tmp_area_code] = tmp_item

                tmp_target_result = sub_target_result

        target_result = list(tmp_target_result.values())

        return target_result

    def city_judgment(self, text):
        '''
        后来增加的城市判断，判断简介里面  坐标后面有没有城市
        :param text: 简介，用户标签
        :return:
        '''
        if '坐标' in text:
            # 坐标  的index，取坐标和坐标后面10个字
            zb_index = text.index('坐标')
            need_text = text[zb_index:zb_index+10]
            for city_i in self.city_list:
                if city_i in need_text:
                    return city_i
        return



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 
# author : LGD
# create_time : 2021/3/16 16:56
# update_time : 2021/3/16 16:56
# copyright : Lavector
# ----------------------------------------------
import threading
from traceback import format_exc


class PyThread(threading.Thread):
    def __init__(self, target, args, kwargs={}):
        super(PyThread, self).__init__()
        self.function = target
        self.args = args
        self.kwargs = kwargs
        self.exit_code = 0
        self.exception = None
        self.exc_traceback = ''

    def run(self):
        try:
            self._run()
        except Exception as e:
            self.exit_code = 1
            self.exception = e
            self.exc_traceback = str(format_exc())

    def _run(self):
        try:
            self.function(*self.args, **self.kwargs)
        except Exception as e:
            raise e


# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 去重
# author : 
# create_time : 2020/6/30 14:43
# update_time : 2020/6/30 14:43
# copyright : Lavector
# ----------------------------------------------
from lavda.analysis.common import parse_multi_column
from lavda.analysis.da_clean.clean_interface import *
from lavda.backexcutor.analysis_interface import *


class DfDistinct(DfProcessInterface, CleanInterface):
    def __init__(self, input_column=None, output_column=None):
        """

        :param input_column: string, 去重字段
        :param output_column: string, 去重字段
        """
        self.result_column = output_column
        self.input_column_list = parse_multi_column(input_column)

    def process(self, data, **params):
        drop_dup_tags = data.duplicated(self.input_column_list, keep='first').astype(int)
        data[self.result_column] = drop_dup_tags

        return data



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 过虑清洗数据
# author : 
# create_time : 2020/7/2 14:19
# update_time : 2020/7/2 14:19
# copyright : Lavector
# ----------------------------------------------
from lavda.analysis.da_filter.filter_interface import RowFilterInterface
from lavda.analysis.da_clean.clean_interface import CleanFlag


class CleanFilter(RowFilterInterface):
    def __init__(self, clean_result_prefix):
        """
        清洗功能输出结果字段的前缀，用以判断哪些是清洗的结果
        :param clean_result_prefix:
        """
        self.clean_result_prefix = clean_result_prefix

    def process(self, data, **params):
        """

        :param data: list(dict())
        :param params:
        :return:
        """
        line_dict, task_meta = data

        # 获取清洗结果的字段名
        clean_output_key = [key for key in line_dict.keys() if key.startswith(self.clean_result_prefix)]

        clean_result = [int(line_dict[key]) for key in line_dict if key in clean_output_key and line_dict[key] is not None]
        clean_result = set(clean_result)

        if int(CleanFlag.YES) in clean_result:
            return True

        return False






#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Author : dada
# @time   : 2020/7/31 11:05
import functools
import pandas as pd
from openpyxl import load_workbook
import xlrd
import re
from lavda.analysis.da_tag.mapping_models import MappingCollect


def filed_services(func):
    @functools.wraps(func)
    def filed(*args,**kwargs):
        data = kwargs.get("df")
        add_head_text = kwargs.get("add_head_text")
        old_columns = list(data.columns)
        data = func(*args,**kwargs)
        new_columns_list = list(data.columns)
        filter_column_list = list(filter(lambda column: column not in old_columns, new_columns_list))
        new_cloumns_dict = {old_name: "{}{}".format(add_head_text, old_name) for old_name in filter_column_list}
        data.rename(columns=new_cloumns_dict, inplace=True)
        return data
    return filed



class VerMappingCollect():
    """
        词表校验
        1.词表，有隐藏sheet，有隐藏字段
        2.词表有空格（非关键词单元格）不能有空格，
        3.连续出现的竖线、单元格开头、结尾不能有坚线、特殊的竖线 如‘｜’就是错误的，比一般的‘|’宽
        :return:
    """

    def __init__(self, path):
        self.path = path

    @property
    def check_hide_sheet(self):
        check_code = 0
        check_msg = '有隐藏表'
        xb = pd.ExcelFile(self.path)
        s = xb.book.sheets()
        for i in s:
            if i.visibility == 1:
                check_code = '有隐藏表'

        return {check_msg: check_code}


    @property
    def check_hide_line(self):
        check_code = 0
        check_msg = '有隐藏行或列'
        wb = load_workbook(self.path)
        for sheet in wb:
            # name = re.findall(r'"(.*?)"', str(sheet))[0]  # sheet name
            for colLetter, colDimension in sheet.column_dimensions.items():
                if colDimension.hidden == True:
                    check_code = '有隐藏行或列'

            for colLetter, colDimension in sheet.row_dimensions.items():
                if colDimension.hidden == True:
                    check_code = '有隐藏行或列'

        return {check_msg: check_code}

    @property
    def check_null(self):
        check_code = 0
        check_msg = '单元格错误-空单元格'
        excel = xlrd.open_workbook(self.path)
        for n in range(excel.nsheets):
            sheet = excel.sheet_by_index(n)
            sheet_name = excel._sheet_names[n]
            if '-' in sheet_name:
                num = int(sheet_name.split('-')[-1])-1
                df = pd.read_excel(self.path, sheet_name=sheet_name,header=None, nrows=num)
                for e in sheet.merged_cells:
                    rl, rh, cl, ch = e
                    base_value = sheet.cell_value(rl, cl)
                    df.iloc[rl:rh, cl:ch] = base_value
                for k in list(df.isnull().any()):
                    if k:
                        check_code = '有空单元格'

        return {check_msg: check_code}

    @property
    def verification(self):

        check_msg = []
        self.ver_map = MappingCollect(self.path)
        for i in self.ver_map:
            j = i.mapping_values
            for veri in [verifi[-1] for verifi in j]:
                if "||" in str(veri):
                    check_msg.append({'有||在里面': veri})
                if '｜' in str(veri):
                    check_msg.append({'有特殊的竖线(｜)在里面': veri})
                if str(veri)[0] == "|" or str(veri[-1]) == "|":
                    check_msg.append({'以|开头或结尾': veri})
                if '(' in veri or ')' in veri or '（' in veri or '）' in veri:
                    veri = veri.replace('）', ')').replace('（', '(')
                    brackets_content = re.findall(r'\((.*?)\)', veri)
                    for e in brackets_content:
                        if '-' in e:
                            check_msg.append({'括号内不能出现减号': veri})
                        if len(e.split('+')) != 2:
                            check_msg.append({'括号里没有加号或加号超过两个': veri})

        error_a = '单元格错误-标签词错误'
        error_a_list = []
        for j in check_msg:
            error_a_list.append(list(j.values())[0])
        check_msg = ','.join(list(set(error_a_list)))
        return {error_a: check_msg}

    @property
    def ver_status(self):
        status = 0
        dic = {}
        if list(self.check_hide_sheet.values())[0] == 0 and list(self.check_null.values())[0] == 0:
            for i in [self.check_hide_sheet, self.check_hide_line, self.check_null, self.verification]:
                if type(i) == dict:
                    dic[list(i.keys())[0]] = list(i.values())[0]
        else:
            for i in [self.check_hide_sheet, self.check_hide_line, self.check_null]:
                if type(i) == dict:
                    dic[list(i.keys())[0]] = list(i.values())[0]
        for key, values in dic.items():
            # if values != 0 and values != '':
            if values not in [0, '']:
                status = 1
                return {status: dic}
        return {status: dic}



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose: 全渠道水军类垃圾检测
# author :
# create_time : 2020/11/30 15:40
# update_time : 2020/11/30 15:40
# copyright : Lavector
# ----------------------------------------------
import re
import emoji
import codecs
import numpy as np
from lavda.analysis.da_clean.water_spam import *
from lavda.util.mysql_util import *
from lavda.analysis.da_tag.diatance_tag import continuous_addition


class WaterRuleInterface(object):
    def predict(self, text, params=None):
        """
        :param text:
        :param params:
        :return:
        """
        pass


# 新加的水军判断,作用于所有渠道
class navy_judgment(WaterRuleInterface):

    def __init__(self):
        # TODO 获取水军列表
        self._load()

    def _load(self):
        self.navy_nickname_list = WaterOrdinaryWays.get_clean('nickname_navy_judgment_word')
        self.navy_content_list = WaterOrdinaryWays.get_clean('content_navy_judgment_word')
        self.navy_introduce_list = WaterOrdinaryWays.get_clean('introduce_navy_judgment_word')

    def judgment(self, data):
        if data['nickname']:
            nickname = str(data['nickname']).lower()
            for navy_nickname_word in self.navy_nickname_list:
                if navy_nickname_word in nickname:
                    return True
        if data['introduce']:
            introduce = str(data['introduce']).lower()
            for navy_introduce_word in self.navy_introduce_list:
                if navy_introduce_word in introduce:
                    return True
        if data['content']:
            text_content = str(data['content']).lower()
            # 判断文本内容是否含有水军判断词
            for navy_content_word in self.navy_content_list:
                if navy_content_word in text_content:
                    return True
            # 〖9.9〗 括号里面是价格
            bracket_content_list = re.findall(r"〖(.*?)〗", text_content)
            if bracket_content_list:
                for bracket_content in bracket_content_list:
                    if '.' in bracket_content and bracket_content.split('.')[0].isnumeric() or bracket_content.isnumeric():
                        return True
            # 判断是否有  满+享
            if continuous_addition(text_content, '满+享', 7):
                return True
            # 判断是否有过多话题##
            if text_content.count('##') >= 15:
                return True
        return False

    def predict(self, data, params=None):
        return self.judgment(data)


class WaterRuleModel(RowInterface, CleanInterface):
    """
    将微博的三种规则和全渠道的一种规则整合在一起判断
    """

    def __init__(self, **params):
        super(WaterRuleModel, self).__init__(CleanFlag.NO)

        self.weibo_rule_list = list()
        self.weibo_rule_list.append(WaterWordRule())
        self.weibo_rule_list.append(WaterRepeatPrefix())
        self.weibo_rule_list.append(WaterHastagRule())
        self.weibo_rule_list.append(WaterForwardMicroblogRule())
        # 新增作用于全渠道的水军判断
        self.all_rule_list = list()
        self.all_rule_list.append(navy_judgment())

    def process(self, data, **params):
        column_name_list = ['channel', 'introduce', 'content', 'nickname']
        for column_name in column_name_list:
            if column_name not in data:
                data[column_name] = ''
            if not data[column_name]:
                data[column_name] = ''
        if data['channel'] == 'weibo':
            for rule in self.weibo_rule_list:
                if rule.predict(data['content']):
                    return CleanFlag.YES

        for _rule in self.all_rule_list:
            if _rule.predict(data):
                return CleanFlag.YES
        return CleanFlag.NO

if __name__ == '__main__':
    dic = {'nickname':'daigo','introduce':'vx:sdfsdf'}
    c = WaterRuleModel()
    print(c.process(dic))



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : HashTag提取
# author : 
# create_time : 2020/7/9 15:11
# update_time : 2020/7/9 15:11
# copyright : Lavector
# ----------------------------------------------
import re
import numpy as np

from lavda.backexcutor.analysis_interface import *


class HastTagExtract(RowInterface):
    def __init__(self, default_value=None, tag_type='en', separator=","):
        """
        :param default_value: , 无数据或非法数据时返回此默认值
        :param tag_type: string, hasttag类型
                        en: 英文
                        ch: 中文
        :param separator: string, 结果分隔符
        """
        super(HastTagExtract, self).__init__(default_value)
        self.default_value = default_value
        self.separator = separator

        if tag_type == "en":
            self.tag_pattern = re.compile('#[^\s]+')
        else:
            self.tag_pattern = re.compile('#.*?#', re.S)

    def process(self, data, params=None):
        if data is None:
            return self.default_value

        data = str(data)
        result = self.tag_pattern.findall(data)
        t_list = [self.separator] * len(result)
        result = "{}".join(result).format(*t_list)

        return result


class HasTagExtractAndFormat(DfProcessInterface):
    """
    hastag tag 数据提取并格式化
    """
    def __init__(self, input_column=None, output_column=None, separator=","):
        self.input_column = input_column
        self.output_column = output_column
        self.separator = separator

    def cross_analysis(self, data, columns):
        """
         做交叉分析。将一个cell中的值，拆分为多行。
        :param data: 数据,dataframe
        :param columns: 要处理的列名,string
        :return:
            拆分好的数据框。因为是将同一个cell中的数据进行拆分，所以要保证一个cell
            拆分后对应的索引不变，这会让我们找到拆分后的数据在原数据中的位置，也方
            便通过统计不同的索引次数，来统计单个值的讨论量
        """

        token = self.separator

        a = data[columns].str.split(token, expand=True)
        b = a.stack()  # 压缩成一列：此时的数据结果变为多层列
        # 重新指定index:drop=true，将变更前的列去掉，默认保留
        c = b.reset_index(level=1, drop=True)
        c1 = c.replace(' ', np.nan).dropna()
        e = c1.rename(columns)  # 为series设置名称，方便join
        f = data.drop(columns, axis=1)  # 删除原始数据中的被分出的列，使用join合并：join默认使用index合并
        g = f.join(e)
        return g

    def process(self, data, **params):
        extractor = HastTagExtract()
        # 找出hashtag后面的值
        data[self.output_column] = data[self.input_column].astype(str).apply(lambda x: extractor.process(x))
        # 替换不必要符号
        data[self.output_column] = data[self.output_column].astype(str).apply(lambda x: re.sub('\[|\]|\'|\ˇ|\#', '', x))
        # 将逗号拆分合并,相同条数复制
        data = self.cross_analysis(data, self.output_column)
        data[self.output_column] = data[self.output_column].apply(lambda x: re.sub(' ', '', x))  # 缩减空格

        return data



if __name__ == "__main__":
    # data = "#SHOTSANDSWATCHES @fentybeauty Cheeks Out. #Cream Blush. 10 shades"
    # extractor = HastTagExtract()
    # result = extractor.process(data)
    # print(result)

    import os
    root_path = "/Users/yuantian/Downloads"
    input_path = os.path.join(root_path, "博主帖子.xlsx")
    output_path = os.path.join(root_path, "博主帖子-result.xlsx")
    from maxda.util.panda_tool import PandaTool
    df_input = PandaTool.read(input_path)

    extractor = HasTagExtractAndFormat("content", "hasttag_result")
    df_result = extractor.process(df_input)

    PandaTool.save(output_path, df_result)

    # data = ['#Georgefloyd', '#restinpower', '#justiceforgeorgefloyd', '#blacklivesmatter']
    # d_lst = [','] * len(data)
    # data = "{}".join(data).format(*d_lst)
    # print(data)


# *-* coding:utf-8 *-*
'''
@author: han
@date:   2020-08-24
'''
import os
import sys
import pandas as pd
from lavda.util.panda_tool import PandaTool
from lavda.util.file_tool import FileTool


global path_root, path_project
path_root = os.getcwd()
path_project = os.path.join(path_root, "data/run_project")
input_data_path = os.path.join(path_project, "1_log")
output_data_path = os.path.join(path_project, "2_merge/result_口红_品牌.xlsx")


def get_file_list(dir, filelist):
    newDir = dir
    if os.path.isfile(dir):
        filelist.append(dir)
    elif os.path.isdir(dir):
        for s in os.listdir(dir):
            # 如果需要忽略某些文件夹，使用以下代码
            if s == ".DS_Store":
                continue
            newDir = os.path.join(dir, s)
            get_file_list(newDir, filelist)
    # 默认顺序排列
    return sorted(filelist, reverse=False)

def save(all_df_result_dict, result_file_path):
    writer = pd.ExcelWriter(result_file_path)
    for label_name in all_df_result_dict:
        df_one_label = all_df_result_dict[label_name]
        df_one_label.to_excel(writer, sheet_name=label_name, index=True)
    writer.save()

# def main():

#     # file_path_friend = FileTool
#     # file_path_all = file_path_friend.get_file_path("C:/Users/zhang/Desktop/口红项目/3、打标结果")
#     file_path_all = get_file_list(input_data_path,filelist=[])
#     dict_df = {}
#     sheetname = 0
#     for file_path in file_path_all:
#         df_statis = PandaTool.read(file_path)
#         dict_df[str(sheetname)] = df_statis
#         sheetname += 1
#     save(dict_df,output_data_path)

class MoreSheetSave():
    def __init__(self):
        pass

    def sheet_save(self,input_data_path,output_data_path):
        file_path_all = get_file_list(input_data_path,filelist=[])
        dict_df = {}
        sheetname = 0
        for file_path in file_path_all:
            df_statis = PandaTool.read(file_path)
            dict_df[str(sheetname)] = df_statis
            sheetname += 1
        save(dict_df,output_data_path)

def main():
    test_friend = MoreSheetSave()
    test_friend.sheet_save(input_data_path,output_data_path)

if __name__ == '__main__':
    main()



# -*- coding:utf-8 -*-
"""
分析处理任务能用接口
"""


class TaskInterface(object):
    def process(self, data, **params):
        """
        高层接口
        任务处理接口
        :param data:任意数据，由底层接口定义
        :param params: dict, 输入参数
        :return: 由底层接口定义
        """
        pass

    def load_config_data(self, **config):
        """
        加载配置文件数据
        :param config:
            格式：配置资源名=> 资源文件名/资源文件完整路径
        :return:
        """
        pass


class RowInterface(TaskInterface):
    """
    核心接口

    """
    def __init__(self, default_value=None):
        self.default_value = default_value

    def get_default_value(self):
        return self.default_value

    def process(self, data, **params):
        """
        以行为单位的数据处理接口
        核心功能接口
        :param data: python基础数据类型，如int, str, dict, 输入数据
        :param params: dict, 输入参数
        :return: 返回数据依赖具体功能
        """
        pass


class DfInterface(TaskInterface):
    """
    高层接口
    实体功能类不要继承此接口
    """
    pass


class DfProcessInterface(DfInterface, TaskInterface):
    def process(self, data, **params):
        """
        核心接口
        基于DataFrame的数据处理接口
        :param data: DataFrame, 输入数据
        :param params: dict, 输入参数
        :return: DataFrame
        """
        pass


class DfStatisInterface(DfInterface, TaskInterface):
    def process(self, data, **params):
        """
        核心接口
        基于DataFrame的数据统计类接口
        不会修改输入数据，只会生成新的统计数据
        :param data: DataFrame
        :param params: dict, 输入参数
        :return: DataFrame 或 dict(name==> DataFrame)， 输出统计结果
        """
        pass


class DfMergeInterface(DfInterface, TaskInterface):
    """
    核心接口
    只有数据合并类才继承此接口
    """
    def process(self, data, **params):
        """

        :param data: 目录/文件名
        :param params:
        :return: DataFrame，合并结果
        """
        pass



# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 
# author : LGD
# create_time : 2021/2/1 16:37
# update_time : 2021/2/1 16:37
# copyright : Lavector
# ----------------------------------------------
import os
import pandas as pd
import xlrd
from typing import List, Optional

from lavda.util.panda_tool import PandaTool
from lavda.analysis.da_tag.label_util import LabelUtil


class LabelExcelMerge(object):
    def process(self, input_file_path, sheet_name=None):
        workbook = xlrd.open_workbook(input_file_path)

        # 获取所有sheet
        # print('打印所有sheet:', workbook.sheet_names())
        if not sheet_name:
            target_sheet = workbook.sheet_by_index(0)  # sheet索引从0开始
        else:
            target_sheet = workbook.sheet_by_name(sheet_name)
        rows_num = target_sheet.nrows
        cols_num = target_sheet.ncols

        # print(target_sheet.merged_cells)
        total_data_list = list()
        for r in range(rows_num):
            line_data_list = list()
            for c in range(cols_num):
                cell_value = target_sheet.row_values(r)[c]
                # print('第%d行第%d列的值：[%s]' % (r, c, sheet2.row_values(r)[c]))
                if cell_value is None or cell_value == '':
                    cell_value = (self.get_merged_cells_value(target_sheet, r, c))
                line_data_list.append(cell_value)

            total_data_list.append(line_data_list)

        return total_data_list

    def get_merged_cells(self, sheet):
        """
        获取所有的合并单元格，格式如下：
        [(4, 5, 2, 4), (5, 6, 2, 4), (1, 4, 3, 4)]
        (4, 5, 2, 4) 的含义为：行 从下标4开始，到下标5（不包含）  列 从下标2开始，到下标4（不包含），为合并单元格
        :param sheet:
        :return:
        """
        return sheet.merged_cells

    def get_merged_cells_value(self, sheet, row_index, col_index):
        """
        先判断给定的单元格，是否属于合并单元格；
        如果是合并单元格，就返回合并单元格的内容
        :return:
        """
        merged = self.get_merged_cells(sheet)
        for rlow, rhigh, clow, chigh in merged:
            if row_index >= rlow and row_index < rhigh:
                if col_index >= clow and col_index < chigh:
                    cell_value = sheet.cell_value(rlow, clow)
                    # print('该单元格[%d,%d]属于合并单元格，值为[%s]' % (row_index, col_index, cell_value))
                    return cell_value
                    break
        return None


MULTI_LABEL_SEPARATOR = "|"


class LabelDictBase(object):
    def __init__(self, id2keyword, id2line, format_data, label_name):
        """
        :param id2keyword: dict, keyword id => keyword 文本
        :param id2line: dict, keyword id => keyword 关联信息，如父类等
        :param format_data: dataframe，标准化的、结构化的词典数据
        :param label_name: string,标签名
        """
        self._id2keyword = id2keyword
        self._id2line = id2line
        self._format_data = format_data
        self._label_name = label_name

        self._columns = list(format_data.columns)

        id2relation = dict()
        for kid in id2line:
            line = [id2line[kid][col] for col in self._columns]
            id2relation[kid] = line
        self._id2relation = id2relation

    @property
    def id2relation(self):
        return self._id2relation

    @property
    def columns(self):
        return self._columns

    @property
    def id2keyword(self):
        return self._id2keyword

    @property
    def id2line(self):
        return self._id2line

    @property
    def label_name(self):
        return self._label_name

    @property
    def format_data(self):
        return self._format_data


class TreeLabelDict(LabelDictBase):
    """
    树形结构词表
    """

    def __init__(self, id2keyword,
                 id2line,
                 format_data,
                 label_name,
                 level_count):
        super(TreeLabelDict, self).__init__(id2keyword, id2line, format_data, label_name)
        self.level_count = level_count


class MappingCollect(object):
    def __init__(self, mappings: List[LabelDictBase]):
        self._mappings = mappings

    @property
    def mappings(self):
        return self._mappings


class LineLabelDict(LabelDictBase):
    """
    以行为单位的标签词表
    """

    def __init__(self, id2keyword,
                 id2line,
                 format_data,
                 label_name):
        super(LineLabelDict, self).__init__(id2keyword, id2line, format_data, label_name)


class PandaLabelDictTool(object):
    LABEL_KEYWORD_COL = "keyword"

    def check_file(self, input_file_path, file_suffix):
        parent_file_path, temp_file_name = os.path.split(input_file_path)
        file_name, extension = os.path.splitext(temp_file_name)
        extension = extension.lower()
        extension = extension.replace(".", "")
        if extension in file_suffix:
            return True
        else:
            raise Exception("unsupport file extension = " + extension)

    def trans(self, total_data_list):
        """
        二维数据置换
        :param total_data_list:
        :return:
        """
        old_max_rows = len(total_data_list)
        old_max_column = 0
        for line in total_data_list:
            current_column = len(line)
            if current_column > old_max_column:
                old_max_column = current_column

        result_data_list = list()
        for i in range(old_max_column):
            sub_data_list = list()
            for j in range(old_max_rows):
                if i < len(total_data_list[j]):
                    sub_data_list.append(total_data_list[j][i])
                else:
                    # TODO 空值填充
                    sub_data_list.append(None)
            result_data_list.append(sub_data_list)

        return result_data_list

    def is_none_value(self, cell_value):
        if pd.isna(cell_value):
            return True
        tmp_cell_value = str(cell_value)
        if not tmp_cell_value:
            return True

        return False

    def drop_none_row(self, total_data_list):
        """
        删除全为None 或空值的行
        :param total_data_list:
        :return:
        """
        result_data_list = list()
        for line in total_data_list:
            is_delete = True
            for cell_value in line:
                if not self.is_none_value(cell_value):
                    is_delete = False
                    break
            if not is_delete:
                result_data_list.append(line)

        return result_data_list

    def drop_none_column(self, total_data_list):
        """
        删除全为None 或空值的行
        :param total_data_list:
        :return:
        """
        delete_column_index = set()
        column_data_list = self.trans(total_data_list)
        for i, line in enumerate(column_data_list):
            is_delete = True
            for cell_value in line:
                if not self.is_none_value(cell_value):
                    is_delete = False
                    break
            if is_delete:
                delete_column_index.add(i)

        result_data_list = list()
        for line in total_data_list:
            sub_list = list()
            for j, cell_value in enumerate(line):
                if j not in delete_column_index:
                    sub_list.append(cell_value)
            result_data_list.append(sub_list)

        return result_data_list

    def check_has_non_value(self, total_data_list):
        for i, line in enumerate(total_data_list):
            for j, cell_value in enumerate(line):
                if self.is_none_value(cell_value):
                    raise Exception("has non value" + ", line=" + str(i) + ", col=" + str(j))

    def drop_end_non_value(self, total_data_list, total_label_level):
        """
        除最后一级标签外， 删除末尾的连续 空值
        :param total_data_list:
        :return:
        """
        result_data = list()
        for i, line in enumerate(total_data_list):
            label_level = i + 1
            if label_level < total_label_level:
                del_list = list()
                data_index = len(line) - 1
                while data_index >= 0:
                    item = line[data_index]
                    if self.is_none_value(item):
                        del_list.append(data_index)
                    else:
                        break
                    data_index -= 1

                if del_list:
                    sub_list = line[0: del_list[len(del_list) - 1]]
                else:
                    sub_list = line
            else:
                sub_list = line
            result_data.append(sub_list)

        return result_data

    def load_all_from_file(self, dict_path) -> MappingCollect:
        """
        加载文件所有的词表
        :param dict_path:
        :return:
        """
        df_sheet = pd.read_excel(dict_path, None)
        sheets = list(df_sheet.keys())

        mapping = list()
        for sheet_name in sheets:
            if "-" in sheet_name:
                tmp_list = sheet_name.split("-")
                tmp_list = [item.strip() for item in tmp_list if item.strip()]
                if len(tmp_list) != 2:
                    raise Exception("unknown sheetname = " + str(sheet_name))
                label_name = tmp_list[0]
                dict_level = int(tmp_list[1])

                label_dict_object = self.load_tree_dict(
                    dict_path,
                    sheet_name=sheet_name,
                    header=False,
                    tree_format=0,
                    total_label_level=dict_level,
                    label_name=label_name,
                    split_text=False)
            else:
                label_name = sheet_name
                label_dict_object = self.load_line_dict(dict_path, label_name=label_name, sheet_name=sheet_name)

            mapping.append(label_dict_object)

        mc = MappingCollect(mapping)

        return mc

    def load_tree_dict(self, input_file_path,
                       sheet_name=None,
                       tree_format=0,
                       header=False,
                       total_label_level=5,
                       label_name=None,
                       split_text=False):
        """

        :param input_file_path: string, 输入文件路径
        :param sheet_name: string, sheet name
        :param tree_format: int, 格式，0：竖形树， 1：横着的树
        :param header: bool, 是否有表头，默认不保留表头信息
        :param total_label_level: int, 标签词典级别
        :param label_name: string, 标签词典级别
        :param split_text: bool, 最后一级是否需要拆分文本
        :return:
        """
        # 1.校验输入文件
        file_suffix = ['xlsx', 'xls']
        self.check_file(input_file_path, file_suffix)

        # 2.处理合并单元格
        excel_merge = LabelExcelMerge()
        dict_array = excel_merge.process(input_file_path, sheet_name=sheet_name)

        # 检查是否有表头
        # TODO 默认不需要表头数据
        if header:
            # 需要去掉表头数据
            dict_array = dict_array[1:]

        # 3.横排的要做转置
        if tree_format == 1:
            # 矩阵转换，横排的要做转换
            dict_array = self.trans(dict_array)

        # 4. 清理
        # 除最后一级外，删除所有行末尾的空值
        dict_array = self.drop_end_non_value(dict_array, total_label_level)
        dict_array = self.drop_none_row(dict_array)
        dict_array = self.drop_none_column(dict_array)

        # 除最后一级外，没能有空值
        if total_label_level > 1:
            self.check_has_non_value(dict_array[0: total_label_level - 1])

        # 以第一行为标准确定完整的列数
        max_column_count = len(dict_array[0])

        # 6. 加载词典信息
        # 全局信息
        cell_pos_dict = dict()
        # 记录最后一级以外的父节点信息
        cell_pos_list = list()

        label_id_dict = dict()
        total_label_id_count = 0

        for i, line in enumerate(dict_array):
            current_level = i + 1
            if current_level > total_label_level:
                current_level = total_label_level
            parent_level = current_level - 1

            sub_pos_list = list()
            cell_pos_list.append(sub_pos_list)
            for j, cell_value in enumerate(line):
                # 超过最大列的不处理
                if j >= max_column_count:
                    continue

                if pd.isna(cell_value):
                    sub_pos_list.append(None)
                    continue
                cell_value = str(cell_value).strip()
                if not cell_value:
                    sub_pos_list.append(None)
                    continue

                if current_level == 1:
                    parent_label_id = -1
                    parent_label_item = None
                else:
                    parent_label_item = cell_pos_list[parent_level - 1][j]
                    if parent_label_item is None:
                        raise Exception("parent is None: line=" + str(i) + ", row" + str(j))
                    parent_label_id = parent_label_item['id']

                # TODO 只对最后一级作拆分
                if current_level == total_label_level and split_text:
                    sub_cell_value_list = cell_value.split("|")
                    sub_cell_value_list = [sub_value.strip() for sub_value in sub_cell_value_list if sub_value.strip()]

                    for sub_cell_value in sub_cell_value_list:
                        pos_value = str(parent_label_id) + "-" + str(current_level) + "-" + sub_cell_value
                        if pos_value in cell_pos_dict:
                            sub_pos_list.append(cell_pos_dict[pos_value])
                            continue

                        new_label_item = dict()
                        new_label_item['id'] = total_label_id_count
                        new_label_item['level'] = current_level
                        new_label_item['parent'] = parent_label_id
                        new_label_item['text'] = sub_cell_value
                        cell_pos_dict[pos_value] = new_label_item
                        sub_pos_list.append(new_label_item)

                        label_id_dict[new_label_item['id']] = new_label_item

                        total_label_id_count += 1

                else:
                    pos_value = str(parent_label_id) + "-" + str(current_level) + "-" + cell_value
                    if pos_value in cell_pos_dict:
                        sub_pos_list.append(cell_pos_dict[pos_value])
                        continue

                    new_label_item = dict()
                    new_label_item['id'] = total_label_id_count
                    new_label_item['level'] = current_level
                    new_label_item['parent'] = parent_label_id
                    new_label_item['text'] = cell_value
                    cell_pos_dict[pos_value] = new_label_item
                    sub_pos_list.append(new_label_item)
                    label_id_dict[new_label_item['id']] = new_label_item

                    total_label_id_count += 1

        # 转在标准格式
        list_data = self._format_dict2list(label_id_dict, total_label_level)
        format_data = self._format_list2df(list_data, label_name, total_label_level)

        format_data = self.clean_keyword(format_data, label_name)
        id2keyword, id2line = self.build_keyword_mapping(format_data, label_name)

        label_dict_object = TreeLabelDict(id2keyword, id2line, format_data, label_name, total_label_level)

        return label_dict_object

    def _format_list2df(self, result, label_name, label_level):
        """

        :param result:
        :param name:
        :param label_level:
                title 格式eg: 消费者(level1) 消费者(level2) 消费者(level3) 消费者(keyword)  消费者(id)
        :return:
        """
        columns = [str(label_name) + "(level%s)" % i for i in range(1, label_level)]
        columns = columns + [PandaLabelDictTool.build_keyword_column(label_name),
                             PandaLabelDictTool.build_id_column(label_name)]
        df = pd.DataFrame(result, columns=columns)

        return df

    def _format_dict2list(self, r_dict, max_level):
        l_tup = sorted(r_dict.items(), key=lambda item: item[0], reverse=True)
        r_list = []

        for k, v in l_tup:
            result = []
            if v["level"] == max_level:
                result.append(v["id"])
                result.append(v["text"])
                self._recursion(r_dict, result, v["parent"])
                r_list.append(result[::-1])
        return r_list

    def _recursion(self, r_dict, result, id):
        ps = r_dict[id]
        text, parent = ps["text"], ps["parent"]
        result.append(text)
        if parent in r_dict:
            self._recursion(r_dict, result, parent)

    def _load_dict_file(self, input_file_path, sheet_name, header=0):
        if isinstance(input_file_path, str):
            parent_file_path, temp_file_name = os.path.split(input_file_path)
            file_name, extension = os.path.splitext(temp_file_name)
            extension = extension.lower()
            if extension.endswith("xlsx") or extension.endswith("xls"):
                df_input = pd.read_excel(input_file_path, header=header, sheet_name=sheet_name)
            elif extension.endswith("csv"):
                df_input = pd.read_csv(input_file_path, header=header)
            else:
                raise Exception("unknown file extension" + input_file_path)
        else:
            # 默认为DataFrame
            df_input = input_file_path

        return df_input

    @staticmethod
    def build_id_column(label_name):
        return str(label_name) + "(id)"

    @staticmethod
    def build_keyword_column(label_name):
        return str(label_name) + "({})".format(PandaLabelDictTool.LABEL_KEYWORD_COL)

    @staticmethod
    def rename_line_dict(df_input, label_name):
        input_columns = df_input.columns
        col_dict = dict()
        for col in input_columns:
            col_dict[col] = str(label_name) + "(" + col + ")"
        df_input.rename(columns=col_dict, inplace=True)
        return df_input

    def load_line_dict(self, input_file_path,
                       sheet_name=None,
                       keyword_column='keyword',
                       label_name=""
                       ):
        """

        :param input_file_path:
        :param label_name: 标签名
        :param keyword_column: 关键词列名

        :param sheet_name: sheet name 列名0000000000000000

        :return:
        """
        df_input = self._load_dict_file(input_file_path, sheet_name)

        df_input = PandaTool.drop_unnamed_column(df_input)

        column_list = PandaTool.get_column(df_input)
        column_list = [i.replace(" ", "").lower() for i in column_list]
        df_input.columns = column_list

        if keyword_column not in column_list:
            raise Exception("can not find {} columns!".format(keyword_column))

        # TODO keyword 再改名
        columns_dict = {keyword_column: PandaLabelDictTool.LABEL_KEYWORD_COL}
        df_input.rename(columns=columns_dict, inplace=True)

        df_input = PandaLabelDictTool.rename_line_dict(df_input, label_name)
        # 生成id
        count = df_input.shape[0]
        id_list = list(range(1, count+1))
        id_column = PandaLabelDictTool.build_id_column(label_name)
        df_input[id_column] = id_list

        df_input = self.clean_keyword(df_input, label_name)
        id2keyword, id2line = self.build_keyword_mapping(df_input, label_name)

        label_dict_object = LineLabelDict(id2keyword, id2line, df_input, label_name)

        return label_dict_object

    def clean_keyword(self, df_format_data, label_name):
        """
        keyword 数据清洗
        :param df_format_data:
        :param label_name:
        :return:
        """
        keyword_column = PandaLabelDictTool.build_keyword_column(label_name)
        # df_format_data[keyword_column] = df_format_data[keyword_column].apply(lambda line: str(line).replace(" ", "").lower())
        df_format_data[keyword_column] = df_format_data[keyword_column].apply(
            LabelUtil.clean_dict_cell)
        return df_format_data

    def build_keyword_mapping(self, df_format_data, label_name):
        """
        构建相关的关联映射关系
        :param df_format_data:
        :param label_name:
        :return:
        """
        dict_data = df_format_data.to_dict(orient="records")
        id_column = PandaLabelDictTool.build_id_column(label_name)
        keyword_column = PandaLabelDictTool.build_keyword_column(label_name)

        id2keyword = dict()
        id2line = dict()
        for line in dict_data:
            id2keyword[line[id_column]] = line[keyword_column]
            id2line[line[id_column]] = line

        return id2keyword, id2line
